@@ -6,7 +6,7 @@
 # Copyright (C) 2002-2006 Yves Perrin <yves.perrin--cern.ch>
 # Copyright (C) 2014, 2016, 2017 Assaf Gordon
 # Copyright (C) 2001-2011, 2013, 2017 Sylvain Beucler
-# Copyright (C) 2013, 2014, 2017-2024 Ineiev
+# Copyright (C) 2013, 2014, 2017-2025 Ineiev
 #
 # This file is part of Savane.
 #
@@ -47,9 +47,8 @@ require_once ('../../include/trackers/general.php');
 
 extract (sane_import ('request', ['digits' => 'transition_id']));
 
-if (!$group_id)
-  exit_no_group ();
-if (!member_check (0, $group_id, 'A'))
+exit_if_no_group ();
+if (!member_check (0, $group_id, MEMBER_FLAGS_ADMIN))
   exit_permission_denied ();
 
 trackers_init ($group_id);
