@@ -48,8 +48,7 @@ require_once ('../include/group.php');
 require_once ('../include/trackers/general.php');
 require_once ('../include/trackers/votes.php');
 
-if (!$group_id)
-  exit_no_group ();
+exit_if_no_group ();
 
 $is_trackeradmin = member_check (0, $group_id, MEMBER_ROLE_TECHNAGER);
 
@@ -98,6 +97,24 @@ function fb_anon_check_failed ($check)
   );
 }
 
+function redirect_postmoditem ()
+{
+  global $item_id, $feedback, $ffeedback;
+  if (!isset ($item_id))
+    return;
+  $params = '';
+  if (isset ($feedback))
+    $params .= '&feedback=' . utils_urlencode ($feedback);
+  if (isset ($ffeedback))
+    $params .= '&ffeedback=' . utils_urlencode ($ffeedback);
+  if ($params === '')
+    $params = $item_id;
+  else
+    $params = "item_id=$item_id$params";
+  # Include tracker item number and feedback in URL, if present.
+  session_redirect ("{$_SERVER['PHP_SELF']}?$params");
+}
+
 extract (sane_import ('request',
   [
     'funcs' => 'func',
@@ -106,7 +123,7 @@ extract (sane_import ('request',
 ));
 extract (sane_import ('post',
   [
-    'true' => ['submitreturn', 'preview'],
+    'true' => ['submitreturn', 'preview', 'form_check_submit'],
     'digits' => ['comment_type_id', 'quote_no', 'new_vote'],
     'pass' => 'comment',
     'preg' =>
@@ -268,7 +285,7 @@ switch ($func)
           trackers_attach_several_files ($item_id, $group_id, $changes);
         # Add new cc if any.
         if ($add_cc)
-          trackers_add_cc ($item_id, $add_cc, $cc_comment, $changes);
+          trackers_add_cc ($item_id, $add_cc, [$cc_comment]);
 
         # Originator Email:
         # "Email address of the person who submitted the item
@@ -284,7 +301,7 @@ switch ($func)
               {
                 if (validate_email ($originator_email))
                   trackers_add_cc (
-                    $item_id, $originator_email, "-SUB-", $changes
+                    $item_id, $originator_email, TRACKERS_CC_SUBMITTED
                   );
                 elseif (!empty ($originator_email))
                   fb (_("Originator E-mail is not valid, thus was not added\n"
@@ -429,7 +446,7 @@ switch ($func)
         if ($add_cc)
           # No notification needs to be sent when a CC is added,
           # it is irrelevant to the item itself.
-          trackers_add_cc ($item_id, $add_cc, $cc_comment, $changes);
+          trackers_add_cc ($item_id, $add_cc, [$cc_comment]);
         if (trackers_data_is_used ("vote"))
           trackers_votes_update ($item_id, $new_vote);
       } # !$process_comment
@@ -475,11 +492,7 @@ switch ($func)
       }
     if (!$process_comment)
       {
-        if (isset ($item_id))
-          {
-            # Include tracker item number in URL, if present.
-            session_redirect ("{$_SERVER['PHP_SELF']}?$item_id");
-          }
+        redirect_postmoditem ();
         $_POST = $_FILES = [];
         $depends_search = $reassign_change_group_search =
         $add_cc = $input_file = $changed = $vfl = $details = $comment = null;
@@ -506,7 +519,7 @@ switch ($func)
 
   case 'delete_cc':
     # Remove a person from the Cc.
-    $changed = trackers_delete_cc ($group_id, $item_id, $item_cc_id, $changes);
+    trackers_delete_cc ($group_id, $item_id, $item_cc_id);
 
     # Unset previous settings and return to the item.
     $depends_search = $reassign_change_group_search = $add_cc = $input_file
@@ -554,6 +567,8 @@ switch ($func)
         else
           $spamscore = 3;
       }
+    if (!isset ($comment_internal_id))
+      exit_missing_param (['comment_internal_id']);
     spam_flag ($item_id, $comment_internal_id, $spamscore, $group_id);
 
     # Return to the item page if it was not the item itself that was
@@ -570,16 +585,21 @@ switch ($func)
         # (the link was not provided).
         exit_permission_denied ();
       }
-
+    if (!isset ($comment_internal_id))
+      exit_missing_param (['comment_internal_id']);
     spam_unflag ($item_id, $comment_internal_id, ARTIFACT, $group_id);
     include '../include/trackers_run/mod.php';
     break;
   case 'viewspam':
     include '../include/trackers_run/mod.php';
     break;
+  case 'subscribe':
+  case 'unsubscribe':
+    include '../include/trackers_run/mod.php';
+    break;
 
-  case 'browse' :
-  default :
+  case 'browse':
+  default:
     include '../include/trackers_run/browse.php';
     break;
 } # switch ($func)
