@@ -52,33 +52,32 @@ require_once ("$dir_name/form.php");
 # specific content/page description.
 # The form should end by #options so it the user does not have to scroll down
 # too much.
-function html_show_displayoptions ($content, $form_opening=0, $submit=0)
+function html_show_displayoptions ($content, $form_opening = 0, $submit = 0)
 {
   return html_show_boxoptions (
     _("Display Criteria"), $content, $form_opening, $submit
   );
 }
 
-function html_show_boxoptions ($legend, $text, $form_opening = 0,
-  $submit = 0
-)
+function html_boxoptionslinkshow ($boxoptionwanted, $legend)
+{
+  $link = "\n<span id='boxoptionslinkshow'>$legend</span>\n";
+  if (!empty ($GLOBALS['sys_ignore_deployed']))
+    return $link;
+  return "\n<script type='text/javascript' src=\"/js/show-hide.php?"
+    . "deploy=$boxoptionwanted&amp;legend=" . utils_urlencode ($legend)
+    . "&amp;box_id=boxoptions&amp;suffix=\"></script>"
+    . "\n<noscript>$link</noscript>\n";
+}
+
+function html_show_boxoptions ($legend, $text, $form_opening = 0, $submit = 0)
 {
   $ret = "\n";
   extract (sane_import ('request', ['true' => 'boxoptionwanted']));
-
-  if ($boxoptionwanted != 1)
-    $boxoptionwanted = 0;
-  else
-    $boxoptionwanted = 1;
-
-  $ret .= "\n<script type='text/javascript' src=\"/js/show-hide.php?"
-    . "deploy=$boxoptionwanted&amp;legend=" . utils_urlencode ($legend)
-    . "&amp;box_id=boxoptions&amp;suffix=\"></script>";
-  $ret .= "\n<noscript>\n<span id='boxoptionslinkshow'>$legend</span>\n"
-    . "</noscript>\n";
-
+  $boxoptionwanted = ($boxoptionwanted != 1)? 0: 1;
+  $ret .= html_boxoptionslinkshow ($boxoptionwanted, $legend);
   $ret .= "<div id='boxoptionscontent'>\n";
-  if ($boxoptionwanted != 1)
+  if ($boxoptionwanted != 1 && empty ($GLOBALS['sys_ignore_deployed']))
     $ret .= "\n<script type=\"text/javascript\" "
       . "src=\"/js/hide-span.php?box_id=boxoptionscontent\"></script>\n";
 
@@ -98,7 +97,7 @@ function html_show_boxoptions ($legend, $text, $form_opening = 0,
 
 function html_hidsubpart_set_deployed ($uniqueid, $deployed)
 {
-  global $is_deployed;
+  global $is_deployed, $sys_ignore_deployed;
 
   # Try to find a deployed value that match the unique id.
   # If found, override the deployed setting (the deployed setting should be
@@ -109,15 +108,18 @@ function html_hidsubpart_set_deployed ($uniqueid, $deployed)
     $deployed = $is_deployed[$uniqueid];
   if ($deployed != 1)
     $deployed = 0;
-  return $deployed;
+  return $deployed && empty ($sys_ignore_deployed);
 }
 
 function html_hidsubpart_js ($deployed, $title, $uniqueid)
 {
- return '<script type="text/javascript" src="/js/show-hide.php?'
-  . "deploy=$deployed&amp;legend=" . utils_urlencode ($title)
-  . "&amp;box_id=hidsubpart&amp;suffix=$uniqueid\"></script>\n"
-  . "\n<noscript>\n<a href=\"#$uniqueid\">$title</a>\n</noscript>";
+  $link = "<a href=\"#$uniqueid\">$title</a>";
+  if (!empty ($GLOBALS['sys_ignore_deployed']))
+    return $link;
+  return '<script type="text/javascript" src="/js/show-hide.php?'
+   . "deploy=$deployed&amp;legend=" . utils_urlencode ($title)
+   . "&amp;box_id=hidsubpart&amp;suffix=$uniqueid\"></script>\n"
+   . "\n<noscript>\n$link\n</noscript>";
 }
 
 # Function to create a an area in the page that can be hidden or shown
@@ -129,12 +131,13 @@ function html_hidsubpart_js ($deployed, $title, $uniqueid)
 # access easily.
 function html_hidsubpart_header ($uniqueid, $title, $deployed = false)
 {
+  global $sys_ignore_deployed;
   $deployed = html_hidsubpart_set_deployed ($uniqueid, $deployed);
   $ret = html_h (
     2, html_hidsubpart_js ($deployed, $title, $uniqueid), $uniqueid
   );
   $ret .= "<div id=\"hidsubpartcontent$uniqueid\">\n";
-  if ($deployed)
+  if ($deployed || !empty ($sys_ignore_deployed))
     return $ret;
   return $ret . '<script type="text/javascript" src="/js/hide-span.php'
     . "?box_id=hidsubpartcontent$uniqueid\"></script>\n";
@@ -145,13 +148,20 @@ function html_hidsubpart_footer ()
   return "\n</div><!-- closing hidsubpart -->\n";
 }
 
-function html_splitpage ($how)
+function html_splitpage ($how, $swap = false)
 {
-  if ($how == 'start' || $how == '1')
-    return "\n<div class='splitright'>\n";
-  if ($how == 'middle' || $how == '2')
-    return  "\n</div><!-- end  splitright -->\n<div class='splitleft'>\n";
-  return "\n</div><!-- end  splitleft -->\n";
+  $formats = [
+    'start' => "\n<div class='%s'>\n",
+    'middle' => "\n</div><!-- end %s -->\n<div class='%s'>\n"
+  ];
+  $formats['1'] = $formats['start'];
+  $formats['2'] = $formats['middle'];
+  $classes = ['splitright', 'splitleft'];
+  if ($swap)
+    $classes = ['splitfirst', 'splitsecond'];
+  if (empty ($formats[$how]))
+    return "\n</div><!-- end {$classes[1]} -->\n";
+  return sprintf ($formats[$how], $classes[0], $classes[1]);
 }
 
 function html_image_dir ($theme, $suffix = null)
@@ -324,22 +334,45 @@ function html_format_feedback ($f)
   return nl2br (markup_basic (utils_specialchars ($f)));
 }
 
-# Print out the feedback.
-function html_feedback ($bottom)
+function html_print_fb_success ($suffix, $class_hide, $script_hide)
 {
-  global $feedback, $ffeedback, $sys_home;
-
-  # Be quiet when there is no feedback.
-  if (!($ffeedback || $feedback))
+  global $feedback, $ffeedback;
+  if (!($feedback && !$ffeedback))
     return;
+  # Only success.
+  $icon = html_image ("bool/ok.png", ['class' => 'feedbackimage']);
+  print "<div id=\"feedback$suffix\" class=\"$class_hide\">"
+    . "<p><span class='feedbacktitle'>$icon<b>"
+    . _("Success:") . "</b></span> $feedback</p></div>\n$script_hide";
+}
 
-  $feedback = html_format_feedback ($feedback);
-  $ffeedback = html_format_feedback ($ffeedback);
 
-  $suffix = '';
-  if ($bottom)
-    $suffix = '_bottom';
+function html_print_fb_error ($suffix, $class_hide, $script_hide)
+{
+  global $feedback, $ffeedback;
+  # Only errors.
+  if (!($ffeedback && !$feedback))
+    return;
+  $icon = html_image ("bool/wrong.png", ['class' => 'feedbackimage']);
+  print "<div id=\"feedback$suffix\" class=\"feedbackerror $class_hide\">"
+    . "<p><span class='feedbackerrortitle'>$icon<b>"
+    . _("Error:") . "</b></span>\n$ffeedback</p></div>\n";
+}
+function html_print_fb_success_error ($suffix, $class_hide, $script_hide)
+{
+  global $feedback, $ffeedback;
+  # Errors and success.
+  if (!($ffeedback && $feedback))
+    return;
+  $icon = html_image ("bool/wrong.png", ['class' => 'feedbackimage']);
+  print "<div id=\"feedback$suffix\" class=\"feedbackerrorandsuccess "
+    . "$class_hide\"><p>"
+    . "<span class='feedbackerrorandsuccesstitle'>$icon<b>". _("Some Errors:")
+    . "</b></span> $feedback<hr />\n$ffeedback</p></div>\n";
+}
 
+function html_get_script_hide ($suffix)
+{
   $script_hide = '<script type="text/javascript" '
     . "src=\"/js/hide-feedback.php?suffix=$suffix\"></script>\n";
 
@@ -352,35 +385,31 @@ function html_feedback ($bottom)
       $class_hide = 'feedback feedback-hide';
       $script_hide = '';
     }
+  return [$script_hide, $class_hide];
+}
 
-  print "<div id=\"feedbackback$suffix\" class='feedbackback'>"
-    . _("Show feedback again") . "</div>\n";
+# Print out the feedback.
+function html_feedback ($bottom)
+{
+  global $feedback, $ffeedback;
+
+  # Be quiet when there is no feedback.
+  if (!($ffeedback || $feedback))
+    return;
+
+  $plain_feedback = $feedback; $plain_ffeedback = $ffeedback;
+  $feedback = html_format_feedback ($feedback);
+  $ffeedback = html_format_feedback ($ffeedback);
+  $suffix = $bottom? '_bottom': '';
+
+  list ($script_hide, $class_hide) = html_get_script_hide ($suffix);
+  print "<div id=\"feedbackback$suffix\" class='feedbackback'></div>\n";
+  html_print_fb_success ($suffix, $class_hide, $script_hide);
+  html_print_fb_error ($suffix, $class_hide, $script_hide);
+  html_print_fb_success_error ($suffix, $class_hide, $script_hide);
   print '<script type="text/javascript" src="/js/show-feedback.php?suffix='
     . "$suffix\"></script>\n";
-
-  $img_ok = html_image ("bool/ok.png", ['class' => 'feedbackimage']);
-  $img_wrong = html_image ("bool/wrong.png", ['class' => 'feedbackimage']);
-  # Only success.
-  if ($feedback && !$ffeedback)
-    print "<div id=\"feedback$suffix\" class=\"$class_hide\">"
-      . '<span class="feedbacktitle">' . $img_ok
-      . _("Success:") . "</span> $feedback</div>\n$script_hide";
-
-  # Only errors.
-  if ($ffeedback && !$feedback)
-    print "<div id=\"feedback$suffix\" class=\"feedbackerror $class_hide\">"
-      . "<span class='feedbackerrortitle'>$img_wrong"
-      . _("Error:") . "</span>\n$ffeedback</div>\n";
-
-  # Errors and success.
-  if ($ffeedback && $feedback)
-    print "<div id=\"feedback$suffix\" class=\"feedbackerrorandsuccess "
-      . "$class_hide\"><span class='feedbackerrorandsuccesstitle'>$img_wrong"
-      . _("Some Errors:") . "</span> $feedback $ffeedback</div>\n";
-
-  # We empty feedback so there will be a bottom feedback only if something
-  # changes.
-  $feedback = $ffeedback = '';
+  $feedback = $plain_feedback; $ffeedback = $plain_ffeedback;
 }
 
 function html_feedback_top ()
@@ -417,11 +446,8 @@ function html_image_attributes ($src, $args)
   if (empty ($args['border']))
     $args['border'] = 0;
 
-  $return = "src=\"$sys_home$base\"";
-
-  foreach ($args as $k => $v)
-    $return .= " $k=\"$v\"";
-
+  $args['src'] = "$sys_home$base";
+  $return = html_attr_str ($args, '"');
   # If there is neither height nor width tag, insert them both.
   if (empty ($args['height']) && empty ($args['width']))
     $return .= ' ' . html_image_get_size ($src, $path);
@@ -457,33 +483,66 @@ function html_label ($for, $title)
   return "<label for='$for'>$title</label>";
 }
 
-function html_dl ($defs, $id = '')
+function html_attr_str ($attr, $quote = "'")
+{
+  $ret = '';
+  if (!is_array ($attr))
+    $attr = ['id' => $attr];
+  foreach ($attr as $k => $v)
+    $ret .= " $k=$quote$v$quote";
+  return $ret;
+}
+
+function html_dl ($defs, $attr = [])
 {
   if (empty ($defs))
     return '';
-  if (!empty ($id))
-    $id = " id='$id'";
-  $ret = "<dl$id>\n";
+  $ret = "<dl" . html_attr_str ($attr) . ">\n";
   foreach ($defs as $k => $v)
     {
-      $id = '';
+      $attr = $dd_attr = '';
       if (is_array ($v))
         {
-          $id = " id='" . $v[0] . "'";
+          $attr = html_attr_str ($v[0]);
+          if (is_array ($v[0]))
+            {
+              if (array_key_exists ('id', $v[0]))
+                unset ($v[0]['id']);
+              $dd_attr = html_attr_str ($v[0]);
+            }
           $v = $v[1];
         }
-      $ret .= "<dt$id>$k</dt>\n  <dd>$v</dd>\n";
+      $ret .= "<dt$attr>$k</dt>\n  <dd$dd_attr>$v</dd>\n";
     }
   return "$ret</dl>\n";
 }
 
+# Transform a list of items in a joined strings with items tagged, e.g.
+# html_join_enclosed (['a', 'b'], ['li', 'code']) produces
+# "<li><code>a</code></li>\n<li><code>b</code></li>".
+function html_join_enclosed ($items, $tags, $separator = "\n", $protect = false)
+{
+  $head = $tail = '';
+  if (!is_array ($tags))
+    $tags = [$tags];
+  foreach ($tags as $t)
+    {
+      $head .= "<$t>";
+      $tail = "</$t>$tail";
+    }
+  $tagged = array_map (
+    function ($x) use ($head, $tail, $protect) {
+      $x = $protect? utils_specialchars ($x): $x;
+      return "$head$x$tail";
+    },
+    $items
+  );
+  return join ($separator, $tagged);
+}
+
 function html_h ($no, $title, $attr = [])
 {
-  $extra = '';
-  if (!is_array ($attr))
-    $attr = ['id' => $attr];
-  foreach ($attr as $k => $v)
-    $extra .= " $k='$v'";
+  $extra = html_attr_str ($attr);
   return "<h$no$extra>$title</h$no>\n";
 }
 
@@ -496,23 +555,17 @@ function html_build_list_table_top (
 )
 {
   $return = '';
-
   if ($table)
     $return = "\n<table class='box'>\n";
-
   $return .= "<tr>\n";
-
   $count = count ($title_arr);
-  if ($links_arr)
-    for ($i = 0; $i < $count; $i++)
-      {
-        $ln = $links_arr[$i]; $ti = $title_arr[$i];
-        $return .= "<th class='boxtitle'>"
-          . "<a class='sortbutton' href=\"$ln\">$ti</a></th>\n";
-      }
-  else
-    for ($i = 0; $i < $count; $i++)
-      $return .= "<th class='boxtitle'>{$title_arr[$i]}</th>\n";
+  for ($i = 0; $i < $count; $i++)
+    {
+      $ti = $title_arr[$i];
+      if ($links_arr)
+        $ti = "<a class='sortbutton' href=\"{$links_arr[$i]}\">$ti</a>";
+      $return .= "<th class='boxtitle'>$ti</th>\n";
+    }
   return "$return</tr>\n";
 }
 
@@ -658,8 +711,8 @@ function html_build_localized_select_box (
 )
 {
   return html_build_select_box_from_arrays (
-    utils_result_column_to_array($result),
-    utils_result_column_to_array($result, 1, true),
+    utils_result_column_to_array ($result),
+    utils_result_column_to_array ($result, 1, true),
     $name, $checked_val, $show_100, $text_100, $show_any, $text_any,
     $show_unknown, $title
   );
