@@ -51,6 +51,44 @@ $dir_name = dirname (__FILE__);
 foreach (['trackers/transition', 'trackers/cookbook', 'utils', 'group'] as $i)
   require_once ("$dir_name/../$i.php");
 
+define ('FIELD_STATUS_ACTIVE', 'A');
+define ('FIELD_STATUS_PERMANENT', 'P');
+define ('FIELD_STATUS_HIDDEN', 'H');
+
+$FIELD_STATUS_SET = [
+  FIELD_STATUS_ACTIVE => 1,
+  FIELD_STATUS_PERMANENT => 1,
+  FIELD_STATUS_HIDDEN => 1
+];
+$FIELD_STATUS_ENABLED = [FIELD_STATUS_ACTIVE, FIELD_STATUS_PERMANENT];
+$FIELD_STATUS_LIST = array_keys ($FIELD_STATUS_SET);
+
+# TRANSLATORS: this is field status.
+$FIELD_STATUS_LABELS = [
+  FIELD_STATUS_ACTIVE => _("Active"),
+  FIELD_STATUS_PERMANENT => _("Permanent"),
+  FIELD_STATUS_HIDDEN => _("Hidden")
+];
+define ('TRANSITION_ALLOWED', 'A');
+define ('TRANSITION_FORBIDDEN', 'F');
+$TRANSITION_STATUS_SET = [
+  TRANSITION_ALLOWED => 1, TRANSITION_FORBIDDEN => 1
+];
+$TRANSITION_STATUS_LIST = array_keys ($TRANSITION_STATUS_SET);
+
+define ('QUERY_SCOPE_SYSTEM', 'S');
+define ('QUERY_SCOPE_GROUP', 'P');
+define ('QUERY_SCOPE_PERSONAL', 'I');
+
+function trackers_data_query_scope_labels ()
+{
+  return [
+    QUERY_SCOPE_GROUP => _('Group'),
+    QUERY_SCOPE_SYSTEM => _('System'),
+    QUERY_SCOPE_PERSONAL => _('Personal')
+  ];
+}
+
 function trackers_data_fetch_all_fields ($group_ids, $group_cond)
 {
   $art_field = ARTIFACT . '_field';
@@ -107,7 +145,7 @@ function trackers_data_get_all_fields ($group_id, $reload = false)
     return;
 
   $BF_USAGE_BY_ID = $BF_USAGE_BY_NAME = [];
-  $group_ids = [GROUP_NONE, $group_id];
+  $group_ids = array_unique ([GROUP_NONE, $group_id]);
   $cond = 'group_id ' . utils_in_placeholders ($group_ids);
   $all_fields = trackers_data_fetch_all_fields ($group_ids, $cond);
   trackers_data_assign_fields ($all_fields, $group_ids);
@@ -149,8 +187,8 @@ function trackers_data_get_notification_settings ($group_id, $tracker)
     FROM {$tracker}_field f, {$tracker}_field_value fv
     WHERE
       fv.group_id = ? AND f.field_name = ?
-      AND fv.bug_field_id = f.bug_field_id AND fv.status != 'H'",
-    [$group_id, $cat_field_name]
+      AND fv.bug_field_id = f.bug_field_id AND fv.status != ?",
+    [$group_id, $cat_field_name, FIELD_STATUS_HIDDEN]
   );
   $settings['nb_categories'] = db_numrows ($result);
   $settings['category'] = [];
@@ -487,17 +525,8 @@ function trackers_data_get_all_report_fields ($report_id = 100)
   return db_numrows ($res) > 0;
 }
 
-# Return all possible values for a select box field.
-# Rk: if the checked value is given then it means that we want this value
-# in the list in any case (even if it is hidden and active_only is requested).
-function trackers_data_get_field_predefined_values (
-  $field, $group_id  = false, $checked  = false, $by_field_id  = false,
-  $active_only  = true
-)
+function trackers_data_predefined_special_fields ($field_name, $group_id)
 {
-  $field_id = ($by_field_id ? $field : trackers_data_get_field_id ($field));
-  $field_name = ($by_field_id ? trackers_data_get_field_name ($field) : $field);
-
   # The "Assigned_to" box requires some special processing,
   # because possible values  are group members) and they are
   # not stored in the trackers_field_value table but in the user_group table.
@@ -506,49 +535,139 @@ function trackers_data_get_field_predefined_values (
 
   if ($field_name == 'submitted_by')
     return trackers_data_get_submitters ($group_id);
+  return null;
+}
 
-  $status_cond = '';
-  $status_cond_params = [];
-
-  if ($active_only)
-    {
-      $status_cond = "status IN (?, ?)";
-      $status_cond_params = [USER_STATUS_ACTIVE, USER_STATUS_PENDING];
-      if ($checked && !is_array ($checked))
-        {
-          $status_cond = "($status_cond OR value_id = ?)";
-          $status_cond_params[] = $checked;
-        }
-      $status_cond = "AND $status_cond ";
-    }
+function trackers_data_predefined_value_order ()
+{
+  global $offset, $max_rows, $limit_predefined_values;
+  $ret = "\nORDER BY `section`, `order_id`, `value` ASC";
+  if (empty ($limit_predefined_values))
+    return $ret;
+  $off = intval ($offset);
+  $rows = $max_rows + 1;
+  return "$ret\nLIMIT $off, $rows";
+}
 
+function trackers_data_predefined_value_sql ($status_cond)
+{
+  static $i = 0;
   # The fields value_id and value must be first in the select statement,
   # because the output is used in the html_build_select_box function.
-
-  # Look for group-specific values first.
-  $sql = "
+  $i++;
+  return "
     SELECT
-      value_id, value, bug_fv_id, bug_field_id, group_id, description,
-      order_id, status
-    FROM " . ARTIFACT . "_field_value
-    WHERE group_id = ? AND bug_field_id = ? $status_cond
-    ORDER BY order_id, value ASC";
-  $res_value = db_execute (
-    $sql, array_merge ([$group_id, $field_id], $status_cond_params)
-  );
-  # If no specific value for this group, then look for default values.
-  if (db_numrows ($res_value) != 0)
-    return $res_value;
+      `value_id`, `value`, `bug_fv_id`, `bug_field_id`, `group_id`,
+      `description`, `order_id`, `status`, $i AS `section`
+    FROM `" . ARTIFACT . "_field_value`
+    WHERE `group_id` = ? AND `bug_field_id` = ? AND $status_cond";
+}
+
+function trackers_data_fetch_active_values ($checked, $prm)
+{
+  $status_cond = "`status` IN (?, ?)";
+  $params = $GLOBALS['FIELD_STATUS_ENABLED'];
+  if ($checked && !is_array ($checked))
+    {
+      $status_cond = "($status_cond OR `value_id` = ?)";
+      $params[] = $checked;
+    }
   return db_execute (
-    $sql, array_merge ([100, $field_id], $status_cond_params)
+    trackers_data_predefined_value_sql ($status_cond)
+    . trackers_data_predefined_value_order (),
+    array_merge ($prm, $params)
   );
 }
 
-function trackers_data_use_field_predefined_values ($field, $group_id)
+function trackers_data_count_special_field_values ($field, $group_id)
+{
+  if ($field == 'assigned_to')
+    return trackers_data_get_technician_count ($group_id);
+
+  if ($field == 'submitted_by')
+    return trackers_data_get_submitter_count ($group_id);
+  return null;
+}
+
+function trackers_data_get_field_value_count ($field, $group_id, $active_only)
+{
+   $ret = trackers_data_count_special_field_values ($field, $group_id);
+   if ($ret !== null)
+     return $ret;
+   $field_id = trackers_data_get_field_id ($field);
+   $table = ARTIFACT . '_field_value';
+   $sql = "
+     SELECT COUNT(`value_id`) AS `cnt` FROM `$table`
+     WHERE `group_id` = ? AND `bug_field_id` = ?";
+   $params = [$group_id, $field_id];
+   if ($active_only)
+     {
+       $sql .= ' AND `status` '
+         . utils_in_placeholders ($GLOBALS['FIELD_STATUS_ENABLED']);
+       $params = array_merge ($params, $GLOBALS['FIELD_STATUS_ENABLED']);
+     }
+   $ret = db_result (db_execute ($sql, $params), 0, 'cnt');
+   if ($ret)
+     return $ret;
+   $params[0] = GROUP_NONE;
+   return db_result (db_execute ($sql, $params), 0, 'cnt');
+}
+
+function trackers_data_fetch_predefined_values (
+  $field_id, $group_id, $checked, $active_only
+)
+{
+  $prm = [$group_id, $field_id];
+  if ($active_only)
+    return trackers_data_fetch_active_values ($checked, $prm);
+  $st = $GLOBALS['FIELD_STATUS_ENABLED'];
+  $params = array_merge ($prm, $st);
+  $sql =
+    '('
+      . trackers_data_predefined_value_sql (
+          '`status` ' . utils_in_placeholders ($st)
+        )
+    . ')';
+  $params = array_merge ($params, $prm, [FIELD_STATUS_HIDDEN]);
+  $sql .= ' UNION '
+    . ' ('
+    . trackers_data_predefined_value_sql ('`status` = ?')
+    . ')';
+  return db_execute (
+    $sql . trackers_data_predefined_value_order (), $params
+  );
+}
+
+# Return all possible values for a select box field.
+# Rk: if the checked value is given then it means that we want this value
+# in the list in any case (even if it is hidden and active_only is requested).
+function trackers_data_get_field_predefined_values (
+  $field, $group_id, $checked  = false, $by_field_id  = false,
+  $active_only  = true
+)
+{
+  $field_id = $by_field_id? $field: trackers_data_get_field_id ($field);
+  $field_name = $by_field_id? trackers_data_get_field_name ($field): $field;
+  $res = trackers_data_predefined_special_fields ($field_name, $group_id);
+  if ($res !== null)
+    return $res;
+  # Look for group-specific values first.
+  $res = trackers_data_fetch_predefined_values (
+      $field_id, $group_id, $checked, $active_only
+    );
+  if (db_numrows ($res))
+    return $res;
+  # If no specific value for this group, then look for default values.
+  return trackers_data_fetch_predefined_values (
+      $field_id, GROUP_NONE, $checked, $active_only
+    );
+}
+
+# Check whether a group field values are the default one or not.
+# If no entry in the database for the relevant field value belong to the group,
+# then it uses default values (fallback).
+function trackers_data_use_field_custom_values ($field, $group_id)
 {
-  # Check whether a group field values are the default one or not.
-  # If no entry in the database for the relevant field value belong to the
-  # group, then it uses default values (fallback).
   $field_id = trackers_data_get_field_id ($field);
 
   $result = db_execute ("
@@ -559,13 +678,23 @@ function trackers_data_use_field_predefined_values ($field, $group_id)
   return db_numrows ($result);
 }
 
+function trackers_data_bf_var ($by_field_id)
+{
+  $var_name = 'BF_USAGE_BY_';
+  if ($by_field_id)
+    return $var_name . 'ID';
+  return $var_name . 'NAME';
+}
+
+function trackers_data_field_exists ($field, $by_field_id = false)
+{
+  $var_name = trackers_data_bf_var ($by_field_id);
+  return array_key_exists ($field, $GLOBALS[$var_name]);
+}
+
 function trackers_data_field_val ($field, $idx, $by_field_id = false)
 {
-  $var_name = "BF_USAGE_BY_";
-  if ($by_field_id)
-    $var_name .= 'ID';
-  else
-    $var_name .= 'NAME';
+  $var_name = trackers_data_bf_var ($by_field_id);
 
   if (!isset ($GLOBALS[$var_name][$field][$idx]))
     return null;
@@ -690,9 +819,10 @@ function trackers_data_is_username_field ($field, $by_field_id = false)
   return ($field == 'assigned_to') || ($field == 'submitted_by');
 }
 
-function trackers_data_is_project_scope ($field, $by_field_id = false)
+function trackers_data_field_is_group_scope ($field, $by_field_id = false)
 {
-  return 'P' == trackers_data_field_val ($field, 'scope', $by_field_id);
+  return FIELD_STATUS_PERMANENT
+    == trackers_data_field_val ($field, 'scope', $by_field_id);
 }
 
 function trackers_data_is_status_closed ($status)
@@ -769,7 +899,10 @@ function trackers_data_get_display_size ($field, $by_field_id = false)
   $v = trackers_data_custom_val ($field, 'display_size', $by_field_id);
   if ($v !== null)
     $val = $v;
-  return explode ('/', $val);
+  $ret = explode ('/', $val);
+  while (count ($ret) < 2)
+    $ret[] = '';
+  return $ret;
 }
 
 # Return the default value associated to a field_name as defined
@@ -912,13 +1045,13 @@ function trackers_data_is_default_value ($item_fv_id)
 
 # Insert a new value for a given field for a given group.
 function trackers_data_create_value (
-  $field, $group_id, $value, $description, $order_id, $status = 'A',
+  $field, $group_id, $value, $description, $order_id, $status = FIELD_STATUS_ACTIVE,
   $by_field_id = false
 )
 {
   if (preg_match ("/^\s*$/", $value))
     {
-      fb (_("Empty field value not allowed"), 0);
+      fb (_("Empty field value not allowed"), 1);
       return;
     }
 
@@ -950,7 +1083,7 @@ function trackers_data_create_value (
       'bug_field_id' => $field_id, 'group_id' => $group_id,
       'value_id' => $value_id, 'value' => $value,
       'description' => $description, 'order_id' => $order_id,
-      'status' => $status
+      'email_ad' => '', 'status' => $status
     ],
     DB_AUTOQUERY_INSERT
   );
@@ -995,12 +1128,12 @@ function trackers_data_report_update_value ($result)
 # Insert a new value for a given field for a given group.
 function trackers_data_update_value (
   $item_fv_id, $field, $group_id, $value, $description, $order_id,
-  $status = 'A'
+  $status = FIELD_STATUS_ACTIVE
 )
 {
   if (preg_match ("/^\s*$/", $value))
     {
-      fb (_("Empty field value not allowed"), 0);
+      fb (_("Empty field value not allowed"), 1);
       return;
     }
   list ($where_cond, $where_cond_params) =
@@ -1017,6 +1150,94 @@ function trackers_data_update_value (
   trackers_data_report_update_value ($result);
 }
 
+function trackers_data_create_canned ($title, $body, $order_id, $group_id)
+{
+  $result = db_autoexecute (
+    ARTIFACT . '_canned_responses',
+    [
+      'group_id' => $group_id, 'title' => $title, 'body' => $body,
+      'order_id' => $order_id,
+    ],
+    DB_AUTOQUERY_INSERT
+  );
+  if ($result)
+    fb (_("Canned bug response inserted"));
+  else
+    fb (_("Error inserting canned bug response"), 1);
+}
+
+function trackers_data_update_canned ($title, $body, $order_id, $item_canned_id)
+{
+  global $group_id;
+  $result = db_autoexecute (
+    ARTIFACT . '_canned_responses',
+    ['title' => $title, 'body' => $body, 'order_id' => $order_id],
+    DB_AUTOQUERY_UPDATE, 'group_id = ? AND bug_canned_id = ?',
+    [$group_id,  $item_canned_id]
+  );
+  if ($result)
+    fb (_("Canned bug response updated"));
+  else
+    fb (_("Error updating canned bug response"), 1);
+}
+
+function trackers_data_transition_exists ($field_id, $from, $to)
+{
+  global $group_id;
+  $res = db_execute ("
+    SELECT `from_value_id`, `to_value_id`, `is_allowed`, `notification_list`
+    FROM `trackers_field_transition`
+    WHERE
+      `group_id` = ? AND `artifact` = ? AND `field_id` = ?
+      AND `from_value_id` = ? AND `to_value_id` = ?",
+   [$group_id, ARTIFACT, $field_id, $from, $to]
+  );
+  return db_numrows ($res);
+}
+
+# Update the existing entry for a transition.
+function trackers_data_update_transition (
+  $field_id, $from, $to, $mail_list, $allowed
+)
+{
+  global $group_id;
+  $result = db_autoexecute (
+    'trackers_field_transition',
+     ['is_allowed' => $allowed, 'notification_list' => $mail_list],
+     DB_AUTOQUERY_UPDATE,
+     'group_id = ? AND artifact = ? AND field_id = ?
+      AND from_value_id = ? AND to_value_id = ?',
+     [$group_id, ARTIFACT, $field_id, $from, $to]
+  );
+
+  if (db_affected_rows ($result) < 1)
+    fb (_("Update of transition failed"), 1);
+  else
+    fb (_("Transition updated"));
+}
+
+# Greate an entry for a transition.
+function trackers_data_create_transition (
+  $field_id, $from, $to, $mail_list, $allowed
+)
+{
+  global $group_id;
+  $result = db_autoexecute (
+    'trackers_field_transition',
+    [
+      'group_id' => $group_id, 'artifact' => ARTIFACT, 'field_id' => $field_id,
+      'from_value_id' => $from, 'to_value_id' => $to, 'is_allowed' => $allowed,
+      'notification_list' => $mail_list,
+    ],
+    DB_AUTOQUERY_INSERT
+  );
+
+  if (db_affected_rows ($result) < 1)
+    fb (_("Insert failed"), 1);
+  else
+    fb (_("New transition inserted"));
+}
+
 # Reset a field settings to its defaults usage (values are untouched). The defaults
 # always belong to group_id GROUP_NONE so make sure we don't delete entries for
 # GROUP_NONE.
@@ -1034,73 +1255,41 @@ function trackers_data_reset_usage ($field_name, $group_id)
   fb (_("Field value successfully reset to defaults."));
 }
 
-# Update a field settings in the trackers_usage_table.
-# Rk: All the show_on_xxx boolean parameters are set to 0 by default because their
-# values come from checkboxes and if not checked the form variable
-# is not set at all. It must be 0 to be ok with the SQL statement.
-function trackers_data_update_usage (
-  $field_name, $group_id, $label, $description, $use_it, $rank, $display_size,
-  $empty_ok, $keep_history, $show_on_add_members = 0, $show_on_add = 0,
-  $transition_default_auth = 'A'
-)
+# See if this field usage exists in the table for this group.
+function trackers_data_have_usage ($table, $field_id, $group_id)
 {
-  # If it's a required field then make sure the use_it flag is true.
-  if (trackers_data_is_required ($field_name))
-    $use_it = 1;
-
-  $field_id = trackers_data_get_field_id ($field_name);
-
-  $lbl = isset ($label)? $label: null;
-  $desc = isset ($description)? $description: null;
-  $disp_size = isset ($display_size)? $display_size: null;
-  $emp_ok = isset ($empty_ok)? $empty_ok: null;
-  $keep_hist = isset ($keep_history)? $keep_history: null;
-
-  foreach (
-    [ 'show_on_add' => 0, 'show_on_add_members' => 0, 'use_it' => 0,
-      'transition_default_auth' => '']
-    as $var => $val
-  )
-    if (!isset ($$var))
-      $$var = $val;
-
-  # See if this field usage exists in the table for this group.
   $result = db_execute ("
-    SELECT bug_field_id FROM " . ARTIFACT . "_field_usage
-    WHERE bug_field_id = ? AND group_id = ?",
+    SELECT `bug_field_id` FROM `$table`
+    WHERE `bug_field_id` = ? AND `group_id` = ?",
     [$field_id, $group_id]
   );
-  $rows = db_numrows ($result);
+  return db_numrows ($result);
+}
 
-  # If it does exist, then update it, else insert a new usage entry
-  # for this field.
-  if ($rows)
+function trackers_data_delete_usage ($table, $field_id, $group_id)
+{
+  db_execute (
+    "DELETE FROM `$table` WHERE `bug_field_id` = ? AND `group_id` = ?",
+    [$field_id, $group_id]
+  );
+}
+
+function trackers_data_execute_usage ($table, $field_id, $group_id, $fields)
+{
+  $rows = trackers_data_have_usage ($table, $field_id, $group_id);
+  if ($rows == 1)
     $result = db_autoexecute (
-      ARTIFACT . '_field_usage',
-      [
-        'use_it' => $use_it, 'show_on_add' => $show_on_add,
-        'show_on_add_members' => $show_on_add_members,
-        'place' => $rank, 'custom_label' => $lbl,
-        'custom_description' => $desc, 'custom_display_size' => $disp_size,
-        'custom_empty_ok' => $emp_ok, 'custom_keep_history' => $keep_hist,
-        'transition_default_auth' => $transition_default_auth
-      ],
-      DB_AUTOQUERY_UPDATE, "bug_field_id = ? AND group_id = ?",
-      [$field_id, $group_id]
+      $table, $fields, DB_AUTOQUERY_UPDATE,
+      "`bug_field_id` = ? AND `group_id` = ?", [$field_id, $group_id]
     );
   else
-    $result = db_autoexecute (
-      ARTIFACT . '_field_usage',
-      [
-        'bug_field_id' => $field_id, 'group_id' => $group_id,
-        'use_it' => $use_it, 'show_on_add' => $show_on_add,
-        'show_on_add_members' => $show_on_add_members,
-        'place' => $rank, 'custom_label' => $lbl,
-        'custom_description' => $desc, 'custom_display_size' => $disp_size,
-        'custom_empty_ok' => $emp_ok, 'custom_keep_history' => $keep_hist,
-        'transition_default_auth' => $transition_default_auth
-      ], DB_AUTOQUERY_INSERT
-    );
+    {
+      if ($rows > 1) # Something went wrong.  Start from a clean page.
+        trackers_data_delete_usage ($table, $field_id, $group_id);
+      $fields['bug_field_id'] = $field_id;
+      $fields['group_id'] = $group_id;
+      $result = db_autoexecute ($table, $fields, DB_AUTOQUERY_INSERT);
+    }
 
   if (db_affected_rows ($result) < 1)
     fb (_("Update of field usage failed."), 1);
@@ -1108,29 +1297,116 @@ function trackers_data_update_usage (
     fb (_("Field usage updated."));
 }
 
+function trackers_data_make_usage_fields (
+  $label, $description, $use_it, $rank, $size, $empty_ok, $keep_history,
+  $on_add_members, $on_add, $allow_transition
+)
+{
+  foreach (
+    [0 => ['on_add', 'on_add_members', 'use_it'], '' => ['allow_transition']]
+    as $val => $vars
+  )
+    foreach ($vars as $var)
+      $$var = isset ($$var)? $$var: $val;
+  # NULL keys are cast to '', so do this in a separate cycle.
+  foreach (['label', 'description', 'size', 'empty_ok', 'keep_history'] as $v)
+    $$v = isset ($$v)? $$v: null;
+  return [
+    'use_it' => $use_it, 'custom_label' => $label, 'place' => $rank,
+    'show_on_add' => $on_add, 'show_on_add_members' => $on_add_members,
+    'custom_keep_history' => $keep_history, 'custom_empty_ok' => $empty_ok,
+    'custom_description' => $description, 'custom_display_size' => $size,
+    'transition_default_auth' => $allow_transition
+  ];
+}
+
+# Update a field settings in the trackers_usage_table.
+function trackers_data_update_usage (
+  $field_name, $group_id, $label, $description, $use_it, $rank, $display_size,
+  $empty_ok, $keep_history, $show_on_add_members, $show_on_add,
+  $transition_default_auth
+)
+{
+  if (trackers_data_is_required ($field_name))
+    $use_it = 1;
+
+  $field_id = trackers_data_get_field_id ($field_name);
+  $fields = trackers_data_make_usage_fields (
+    $label, $description, $use_it, $rank, $display_size,
+    $empty_ok, $keep_history, $show_on_add_members, $show_on_add,
+    $transition_default_auth
+  );
+  $table = ARTIFACT . '_field_usage';
+  trackers_data_execute_usage ($table, $field_id, $group_id, $fields);
+}
+
+function trackers_data_get_technician_count ($group_id)
+{
+  $uids = array_keys (member_get_group_members ($group_id));
+  $uids = member_check_array ($uids, $group_id, 1);
+  return count ($uids);
+}
+
+function trackers_data_limit_technicians (&$uids)
+{
+  global $offset, $max_rows, $limit_predefined_values;
+  if (empty ($limit_predefined_values))
+    return '';
+  $off = intval ($offset);
+  $rows = intval ($max_rows);
+  array_push ($uids, $off, $rows);
+  return " LIMIT ?, ?";
+}
+
 # Get a list of technicians for a tracker.
 function trackers_data_get_technicians ($group_id)
 {
   # Get list of members.
   $uids = array_keys (member_get_group_members ($group_id));
   $uids = member_check_array ($uids, $group_id, 1);
-  $sql = "SELECT user_id, user_name FROM user WHERE ";
+  $sql = "
+    SELECT `user_id` AS `value_id`, `user_name` AS `value`,
+      0 AS `bug_fv_id`, ? AS `group_id`, `realname` AS `description`,
+      0 AS `order_id`, '" . FIELD_STATUS_PERMANENT . "' AS `status`
+    FROM `user` WHERE ";
   if (empty ($uids))
     $sql .= 'NULL'; # Return a valid (but empty) result set.
   else
-    $sql .= 'user_id ' . utils_in_placeholders ($uids);
-  $sql .= " ORDER BY user_name";
+    $sql .= '`user_id` ' . utils_in_placeholders ($uids);
+  $sql .= " ORDER BY `user_name`";
+  array_unshift ($uids, $group_id);
+  $sql .= trackers_data_limit_technicians ($uids);
   return db_execute ($sql, $uids);
 }
 
-function trackers_data_get_submitters ($group_id = false)
+function trackers_data_get_submitter_count ($group_id)
 {
+  $art = ARTIFACT;
+  $res = db_execute ("
+    SELECT COUNT(DISTINCT `user_id`) AS `cnt`
+    FROM `user` JOIN `$art` `a` ON `user`.`user_id` = `a`.`submitted_by`
+    WHERE `a`.`group_id` = ?", [$group_id]
+  );
+  if (!db_numrows ($res))
+    return 0;
+  return db_result ($res, 0, 'cnt');
+}
+
+function trackers_data_get_submitters ($group_id)
+{
+  global $offset, $max_rows;
+  $off = intval ($offset);
+  $rows = intval ($max_rows);
+
   $art = ARTIFACT;
   return db_execute ("
-    SELECT DISTINCT user.user_id, user.user_name
-    FROM user, $art a
-    WHERE user.user_id = a.submitted_by AND a.group_id = ?
-    ORDER BY user.user_name", [$group_id]
+    SELECT DISTINCT `user_id` AS `value_id`, `user_name` AS `value`,
+      0 AS `bug_fv_id`, ? AS `group_id`, `realname` AS `description`,
+      0 AS `order_id`, ? AS `status`
+    FROM `user` JOIN `$art` `a` ON `user`.`user_id` = `a`.`submitted_by`
+    WHERE `a`.`group_id` = ?
+    ORDER BY `user`.`user_name` LIMIT ?, ?",
+    [$group_id, FIELD_STATUS_PERMANENT, $group_id, $off, $rows]
   );
 }
 
@@ -1467,8 +1743,7 @@ function update_details ($details, $item_id, $group_id, &$vfl, &$changes)
     trackers_data_get_value (
       'comment_type_id', $group_id, $vfl['comment_type_id']
     );
-  if (user_isloggedin () && !user_get_preference ("skipcc_postcomment"))
-    trackers_add_cc ($item_id, user_getid (), "-COM-", $changes);
+  trackers_data_update_user_specific ($item_id, TRACKERS_CC_COMMENTED);
   return 1;
 }
 
@@ -1537,8 +1812,7 @@ function update_changes_in_db ($item_id, $group_id, $upd_list, $change_exists)
   $res = db_autoexecute (ARTIFACT, $upd_list, DB_AUTOQUERY_UPDATE,
     "bug_id = ? AND group_id = ?", [$item_id, $group_id]
   );
-  if (user_isloggedin () && !user_get_preference ("skipcc_updateitem"))
-    trackers_add_cc ($item_id, user_getid (), "-UPD-");
+  trackers_data_update_user_specific ($item_id, TRACKERS_CC_UPDATED);
 
   if (!db_affected_rows ($res))
     exit_error (_("Item update failed"));
@@ -1660,11 +1934,7 @@ function handle_update (
     }
   if (count ($extra_addr))
     $extra_addresses = join (', ', $extra_addr);
-
-  # If we are on the cookbook, store related links.
-  if (ARTIFACT == 'cookbook')
-    cookbook_handle_update ($item_id, $group_id);
-
+  cookbook_handle_update ($item_id, $group_id);
   return update_changes_in_db ($item_id, $group_id, $upd_list, $change_exists);
 }
 } # namespace trackers_data\handle_update {
@@ -2004,101 +2274,42 @@ function trackers_data_update_dependent_items ($depends_on, $item_id, $artifact)
   );
 }
 
-function trackers_data_create_item ($group_id, $vfl, &$extra_addresses)
+# Find transitions from 'none'.
+function trackers_data_create_item_trans (
+  $field, $value, $field_transition, $trans_id, $changes
+)
 {
-  # We don't force them to be logged in to submit a bug.
-  unset ($ip);
-  if (!user_isloggedin ())
-    $user = 100;
-  else
-    $user = user_getid ();
-
-  # Make sure required fields are not empty.
-  if (!trackers_check_empty_fields ($vfl))
-    # In such circumstances, we reprint the form
-    # highligthing missing fields.
-    # (It is important that trackers_check_empty_fields set the global var
-    # previous_form_bad_fields.)
-    return false;
-
-  # Finally, create the bug itself.
-  # This SQL query only sets up the values for fields used by
-  # this group. For other unused fields we assume that the DB will set
-  # up an appropriate default value (see bug table definition).
-
-  # Extract field transition possibilities:
-  $field_transition = trackers_transition_get_update ($group_id);
-  # We shall store in an array the transition_id accepted, to check
-  # other field updates.
-  $field_transition_accepted = $changes = [];
-
-  # Build the variable list of fields and values.
-  # We must add open/closed by ourselves, as it is missing from the
-  # form for obvious reasons while automatic transitions may rely on its
-  # presence.
-  $vfl['status_id'] = '1';
-  reset ($vfl);
-  $insert_fields = [];
-  $field_transition_id = '';
-  foreach ($vfl as $field => $value)
+  $trans_cc = '';
+  $cc = $id = [];
+  foreach (['any', '100'] as $old_val)
     {
-      if (trackers_data_is_special ($field))
+      list ($cc, $id) =
+        trackers_data\handle_update\transition_id_cc (
+          $field, $old_val, $value, $field_transition, $changes
+        );
+      if (empty ($id))
         continue;
+      $trans_cc = array_shift ($cc);
+      $trans_id = array_shift ($id);
+      break;
+    }
+  return [$trans_cc, $trans_id];
+}
 
-      # If value is the special string default,*
-      # take the default from the database.
-      if ($value == "!unknown!")
-        continue;
+function trackers_data_create_user_id ()
+{
+  return user_isloggedin ()? user_getid (): 100;
+}
 
-      # COPIED from handle_update transition code, with one exception:
-      # old_value is equal to "none".
-      # Handle field transitions checks/changes.
-      $field_id = trackers_data_get_field_id ($field);
-      $field_transition_cc = '';
-      if (array_key_exists ($field_id, $field_transition)
-          # First check basic transition;
-          # check multiple transition, override other transition.
-          && (array_key_exists ("100", $field_transition[$field_id])
-              || array_key_exists ("any", $field_transition[$field_id])))
-        {
-           $ft = $field_transition[$field_id];
-           if (array_key_exists ("any", $ft)
-               && array_key_exists ($value, $ft["any"]))
-             {
-               $field_transition_cc = $ft["any"][$value]['notification_list'];
-               if (trackers_data\transition_new_change ($changes, $field))
-                 $field_transition_id = $ft["any"][$value]['transition_id'];
-             }
-           elseif (array_key_exists ("100", $ft)
-                    && array_key_exists ($value, $ft["100"]))
-             {
-               $field_transition_cc = $ft["100"][$value]['notification_list'];
-               if (trackers_data\transition_new_change ($changes, $field))
-                 $field_transition_id = $ft["100"][$value]['transition_id'];
-             }
-        }
-
-      if (trackers_data_is_text ($field))
-        $value = utils_specialchars ($value);
-      elseif (trackers_data_is_date_field ($field))
-        list ($value, $ok) = utils_date_to_unixtime ($value);
-
-      $insert_fields[$field] = $value;
-
-      trackers_data\update_changes ($field, $group_id, '', $value, $changes);
-      # Register transition id.
-      $field_transition_accepted[] = $field_transition_id;
-
-      if ($field_transition_cc)
-        $extra_addresses .= $field_transition_cc;
-    } # foreach ($vfl as $field => $value)
-
-  # Get the default spamscore.
+# Add all special fields that were not handled in the general
+# foreach ($vfl as $field => $value) cycle.
+function trackers_data_finalize_insert_fields ($vfl, $group_id, &$insert_fields)
+{
+  $user = trackers_data_create_user_id ();
   $spamscore = spam_get_user_score ($user);
-  if ($spamscore > 4)
+  if ($spamscore >= 5)
     $vfl['summary'] = "[SPAM] " . $vfl['summary'];
 
-  # Add all special fields that were not handled in the previous block.
   $insert_fields['close_date'] = 0;
   $insert_fields['group_id'] = $group_id;
   $insert_fields['submitted_by'] = $user;
@@ -2109,53 +2320,105 @@ function trackers_data_create_item ($group_id, $vfl, &$extra_addresses)
   $insert_fields['ip'] = '127.0.0.1';
   for ($i = 1; $i <= 10; $i++)
     $insert_fields["custom_ta$i"] = '';
+  return $spamscore;
+}
 
-  # Actually insert the entry.
+function trackers_data_create_try_insert ($insert_fields)
+{
   $result = db_autoexecute (ARTIFACT, $insert_fields, DB_AUTOQUERY_INSERT);
   $item_id = db_insertid ($result);
-
-  if (!$item_id)
+  if ($item_id)
     {
-      fb (
-        _("New item insertion failed, please report this issue to the\n"
-          . "administrator"),
-        1
+      # TANSLATORS: the first argument is tracker type (like sr, bug or recipe);
+      # the second argument is item id (number).
+      $msg = sprintf (
+        _('New item posted (%1$s #%2$s)'),
+        utils_get_tracker_prefix (ARTIFACT), $item_id
       );
-      return false;
+      fb ($msg);
+      return $item_id;
     }
+  $msg = _("New item insertion failed, please report this issue to the\n"
+    . "administrator");
+  fb ($msg, 1);
+  return false;
+}
 
-  # TANSLATORS: the first argument is tracker type (like sr, bug or recipe)
-  # the second argument is item id (number).
-  $msg = sprintf (
-    _('New item posted (%1$s #%2$s)'),
-    utils_get_tracker_prefix (ARTIFACT), $item_id
+function trackers_data_create_process_field (
+  $field, $value, $field_transition, &$changes, &$insert_fields,
+  &$field_transition_accepted, &$extra_addresses, &$trans_id, $group_id
+)
+{
+  if (trackers_data_is_special ($field))
+    return;
+
+  # If value is the special string, take the default from the database.
+  if ($value == "!unknown!")
+    return;
+
+  list ($trans_cc, $trans_id) = trackers_data_create_item_trans (
+    $field, $value, $field_transition, $trans_id, $changes
   );
-  fb ($msg);
 
-  # Register the spam score.
+  if (trackers_data_is_text ($field))
+    $value = utils_specialchars ($value);
+  elseif (trackers_data_is_date_field ($field))
+    list ($value, $ok) = utils_date_to_unixtime ($value);
+
+  $insert_fields[$field] = $value;
+  trackers_data\update_changes ($field, $group_id, '', $value, $changes);
+  $field_transition_accepted[] = $trans_id;
+  if ($trans_cc)
+    $extra_addresses .= $trans_cc;
+}
+
+function trackers_data_create_process_spam ($item_id, $group_id, $spamscore)
+{
+  $user = trackers_data_create_user_id ();
   spam_set_item_default_score ($item_id, '0', ARTIFACT, $spamscore, $user);
-
-  # Add to spamcheck queue, if necessary (will temporary set the spamscore to
-  # 5, if necessary).
-  # Useless, if already considered to be spam.
   if ($spamscore < 5)
     spam_add_to_spamcheck_queue ($item_id, 0, ARTIFACT, $group_id, $spamscore);
+}
 
-  # If we are on the cookbook, store related links.
-  if (ARTIFACT == 'cookbook')
-   cookbook_handle_update ($item_id, $group_id);
+function trackers_data_update_user_specific ($item_id, $mark)
+{
+  if (!user_isloggedin ())
+    return;
+  $user = user_getid ();
+  trackers_add_cc ($item_id, $user, $mark);
+  member_assign_uidNumber ($user);
+}
 
-  # Now we run transitions other fields update. This function does check
-  # what already changed and that we shan't automatically update.
+function trackers_data_create_item ($group_id, $vfl, &$extra_addresses)
+{
+  # Make sure required fields are not empty.
+  if (!trackers_check_empty_fields ($vfl))
+    return false;
+
+  $field_transition = trackers_transition_get_update ($group_id);
+  # Store the transition_id accepted in an array to check other field updates.
+  $field_transition_accepted = $changes = $insert_fields = [];
+
+  $vfl['status_id'] = '1';
+  reset ($vfl);
+  $trans_id = '';
+  foreach ($vfl as $field => $value)
+    trackers_data_create_process_field (
+      $field, $value, $field_transition, $changes, $insert_fields,
+      $field_transition_accepted, $extra_addresses, $trans_id, $group_id
+    );
+  $spamscore =
+    trackers_data_finalize_insert_fields ($vfl, $group_id, $insert_fields);
+  if (false === ($item_id = trackers_data_create_try_insert ($insert_fields)))
+    return false;
+
+  trackers_data_create_process_spam ($item_id, $group_id, $spamscore);
+  cookbook_handle_update ($item_id, $group_id);
   trackers_transition_update_item (
     $item_id, $field_transition_accepted, $changes
   );
 
-  # Add the submitter in CC
-  # (currently, no option to avoid this, but we could make this a notif
-  # configuration option, if wanted).
-  if (user_isloggedin ())
-    trackers_add_cc ($item_id, user_getid (), "-SUB-");
+  trackers_data_update_user_specific ($item_id, TRACKERS_CC_SUBMITTED);
   return $item_id;
 }
 
@@ -2217,16 +2480,15 @@ function trackers_data_get_canned_responses ($group_id)
 
 function trackers_data_get_reports ($group_id, $user_id)
 {
-  $system_scope = 'S'; $group_scope = 'P'; $personal_scope = 'I';
   $sql = "
     SELECT report_id, name, scope FROM " . ARTIFACT . "_report
     WHERE
       (group_id = ? AND scope = ?) OR scope = ?";
-  $params = [$group_id, $group_scope, $system_scope];
+  $params = [$group_id, QUERY_SCOPE_GROUP, QUERY_SCOPE_SYSTEM];
   if ($user_id)
     {
       $sql .= "\n      OR (user_id = ? AND group_id = ? AND scope = ?)";
-      array_push ($params, $user_id, $group_id, $personal_scope);
+      array_push ($params, $user_id, $group_id, QUERY_SCOPE_PERSONAL);
     }
 
   # List system-wide queries first, then group-defined queries,
@@ -2239,11 +2501,11 @@ function trackers_data_get_reports ($group_id, $user_id)
 function trackers_data_report_is_editable ($row, $uid, $is_admin)
 {
   $scope = strtoupper ($row['scope']);
-  if ($scope == 'S' && user_is_super_user ())
+  if ($scope == QUERY_SCOPE_SYSTEM && user_is_super_user ())
     return true;
-  if ($scope == 'I' && $uid == $row['user_id'])
+  if ($scope == QUERY_SCOPE_PERSONAL && $uid == $row['user_id'])
     return true;
-  return $scope == 'P' && $is_admin;
+  return $scope == QUERY_SCOPE_GROUP && $is_admin;
 }
 
 function trackers_data_get_editable_reports ($group_id, $is_admin)
