@@ -7,7 +7,7 @@
 # Copyright (C) 2002-2006 Yves Perrin <yves.perrin--cern.ch>
 # Copyright (C) 2014, 2016, 2017 Assaf Gordon
 # Copyright (C) 2001-2011, 2013, 2017 Sylvain Beucler
-# Copyright (C) 2013, 2014, 2017-2024 Ineiev
+# Copyright (C) 2013, 2014, 2017-2025 Ineiev
 #
 # This file is part of Savane.
 #
@@ -238,7 +238,7 @@ function format_member_icon ($poster_id, $group_id)
 {
   if ($poster_id == 100)
     return ['', ''];
-  if (member_check ($poster_id, $group_id, 'A'))
+  if (member_check ($poster_id, $group_id, MEMBER_FLAGS_ADMIN))
     {
       if ($group_id == $GLOBALS['sys_group_id'])
         return ["site-admin", _("Site Administrator")];
@@ -382,7 +382,7 @@ function format_details (
   format_set_comment_ids ($data, $item_id, $ascii);
   $comment_order = format_enforce_comment_order ($data, $ascii);
   $out = format_details_header ($ascii);
-  $is_admin = member_check (0, $group_id, 'A');
+  $is_admin = member_check (0, $group_id, MEMBER_FLAGS_ADMIN);
   foreach ($data as $i => $entry)
     {
       if ($ascii)
@@ -494,25 +494,48 @@ function format_item_change_separator ()
   return "\n    _______________________________________________________\n\n";
 }
 
+function format_attachment ($file, $public)
+{
+  foreach (['size', 'name', '_id'] as $k)
+    if (array_key_exists ("file$k", $file))
+     {
+        $k1 = str_replace ('_', '', $k);
+        $file[$k1] = $file["file$k"];
+     }
+  $ret = sprintf ("Name: %-30s Size: %s\n",
+    $file['name'], utils_human_readable_size (intval ($file['size']))
+  );
+  $description = '';
+  if (!empty ($file['description']))
+    $description = $file['description'];
+  return [$ret . format_attachment_link ($file, $public), $description];
+}
+
+function format_file_list ($changes, $public)
+{
+  $description = $ret = '';
+  foreach ($changes['attach'] as $file)
+    {
+      list ($entry, $description) = format_attachment ($file, $public);
+      $ret .= $entry;
+    }
+  return [$ret, $description];
+}
+
 function format_change_files ($out, $changes, $item_group, $public)
 {
   if (empty ($changes['attach']))
     return $out;
-  $out_att = "Additional Item Attachment";
+  list ($attachments, $description) = format_file_list ($changes, $public);
+  $out_att = 'Additional Item Attachment';
   if ($out)
     $out .= format_item_change_separator ();
   else
     $out_att .= ", $item_group";
   $out_att .= ":\n\n";
-
-  foreach ($changes['attach'] as $file)
-    {
-      $out_att .= sprintf ("File name: %-30s Size: %s\n",
-        $file['name'], utils_filesize (null, intval ($file['size']))
-      );
-      $out_att .= format_attachment_link ($file, $public);
-    }
-  $out .= $out_att;
+  if (!empty ($description))
+    $out_att = "$out_att$description\n\n";
+  $out .= "$out_att$attachments";
   if ($public)
     $out .= format_file_agpl_notice ();
   return $out;
@@ -589,8 +612,7 @@ function format_item_file_header ($list_is_empty, $ascii)
     {
       if ($ascii)
         return '';
-      return
-        '<span class="warn">' . _("No files currently attached") . '</span>';
+      return '<p>' . _("This item currently has no attached files.") . "</p>\n";
     }
   if ($ascii)
     return "    _______________________________________________________\n"
@@ -598,16 +620,6 @@ function format_item_file_header ($list_is_empty, $ascii)
   return $HTML->box_top (_("Attached Files"), '', 1);
 }
 
-function format_item_attachment_ascii ($row, $href, $public)
-{
-  $ret = "\n-------------------------------------------------------\n";
-  $ret .= sprintf ("Name: %s  Size: %s",
-    $row['filename'], utils_filesize (null, intval ($row['filesize']))
-  );
-  if ($public)
-    $ret .= "\n<$href>";
-  return $ret;
-}
 function format_item_file_details ($row, $href)
 {
   $lnk = "<a href=\"$href\">file #{$row['file_id']}: &nbsp;";
@@ -616,7 +628,8 @@ function format_item_file_details ($row, $href)
     $lnk . utils_specialchars ($row['filename']) . '</a>',
     utils_user_link ($row['user_name'])
   );
-  $out .= ' <span class="smaller">(' . utils_filesize (null, $row['filesize']);
+  $out .= ' <span class="smaller">('
+    . utils_human_readable_size ($row['filesize']);
   if ($row['filetype'])
     $out .= ' - ' . $row['filetype'];
   if ($row['description'])
@@ -642,19 +655,23 @@ function format_list_item_files ($result, $may_delete, $ascii, $public)
   $out = '';
   for ($i = 0; $row = db_fetch_array ($result); $i++)
     {
+      $r = $row;
       $url = format_file_url (
         ['id' => $row['file_id'], 'name' => $row['filename']], $public
       );
       if ($ascii)
-        $out .= format_item_attachment_ascii ($row, $url, $public);
+        {
+          list ($entry, $description) = format_attachment ($row, $public);
+          $out .= $entry;
+        }
       else
         $out .= '<div class="' . utils_altrow ($i) . '">'
           . format_item_attachment_html ($row, $may_delete, $url) . "</div>\n";
     }
-  if ($ascii && !empty ($row['description']))
+  if ($ascii && !empty ($description))
     # The description is common for all files in the original submission,
     # so only write it once.
-    $out = $row['description'] . $out;
+    $out = "$description\n\n$out";
   if ($ascii && $public)
     $out .= "\n" . format_file_agpl_notice ();
   return $out;
@@ -696,16 +713,21 @@ function format_item_cc_list_email ($row)
 
 function format_item_cc_list_comment ($row)
 {
-  $vot = _('Voted in favor of this item');
   $com_arr = [
-    '-SUB-' => _('Submitted the item'), '-COM-' => _('Posted a comment'),
-    '-UPD-' => _('Updated the item'), '-VOT-' => $vot,
-    'Voted in favor of this item' => $vot
+    TRACKERS_CC_SUBMITTED => _('Submitted the item'),
+    TRACKERS_CC_COMMENTED => _('Posted a comment'),
+    TRACKERS_CC_UPDATED => _('Updated the item'),
+    TRACKERS_CC_VOTED => _('Voted in favor of this item'),
+    TRACKERS_CC_SUBSCRIBED => _('Subscribed to discussion')
   ];
 
   $comment = $row['comment'];
   if (array_key_exists ($comment, $com_arr))
     return $com_arr[$comment];
+  # Remove the space that trackers_extract_cc_comment() may have added.
+  $regex = '/^' . TRACKERS_CC_AFFIX . TRACKERS_CC_REGEX . '$/';
+  if (preg_match ($regex, $comment))
+    return substr ($comment, strlen (TRACKERS_CC_AFFIX));
   return $comment;
 }
 
@@ -761,6 +783,7 @@ function format_item_cc_list_entry ($row, $item_id, $user_data, $i)
 function format_item_cc_list ($item_id, $group_id)
 {
   global $HTML;
+  $cc_list = [];
   $result = trackers_data_get_cc_list ($item_id);
   $n = db_numrows ($result);
   $out = format_item_cc_list_header ($n);
@@ -769,8 +792,43 @@ function format_item_cc_list ($item_id, $group_id)
   $user_data = format_item_cc_list_user_data ($group_id);
   $i = 0;
   while ($row = db_fetch_array ($result))
-    $out .= format_item_cc_list_entry ($row, $item_id, $user_data, $i++);
+    {
+      $out .= format_item_cc_list_entry ($row, $item_id, $user_data, $i++);
+      $cc_list[$row['email']] = $row;
+    }
   $out .= $HTML->box_bottom (1);
-  return $out;
+  return [$out, $cc_list];
+}
+
+function format_item_subscribe ($item_id, $group_id)
+{
+  $name = user_getname ();
+  return '<p>' . _('You are not subscribed to this discussion.') . ' '
+    . form_submit (_("Subscribe"), "submit")  . "</p>\n";
+}
+
+function format_item_unsubscribe ($item_id, $group_id)
+{
+  return '<p>' . _('You are subscribed to this discussion.') . ' '
+    . form_submit (_("Unsubscribe"), "submit") . "</p>\n";
+}
+
+function format_item_subscription ($item_id, $group_id, $cc_list)
+{
+  if (!user_isloggedin ())
+    return;
+  $subscribed = false;
+  foreach ([user_getid (), user_getname (), user_getemail ()] as $email)
+    if (!empty ($cc_list[$email]))
+      $subscribed = true;
+  $ret = form_tag () . form_hidden ([
+    'group_id' => $group_id, 'item_id' => $item_id,
+    'func' => $subscribed? 'unsubscribe': 'subscribe',
+  ]);
+  if ($subscribed)
+    $ret .= format_item_unsubscribe ($item_id, $group_id);
+  else
+    $ret .= format_item_subscribe ($item_id, $group_id);
+  return "$ret</form>\n";
 }
 ?>
