@@ -125,13 +125,10 @@ unset ($url_params['history_date']);
 # Make safe for inclusion in an URL (replace quotes with dots).
 function sanitize_val ($x)
 {
-  if (is_string ($x))
-    {
-      $x = strtr ($x, '"\'', '..');
-      return $x;
-    }
   if (is_numeric ($x)) # Numbers are sane.
     return $x;
+  if (is_string ($x))
+    return $x; # Sanitize strings later.
   return 'xxx'; # Some queer type fed.
 }
 
@@ -155,7 +152,7 @@ function sanitize_value_id ($value_id)
   return $ret;
 }
 
-# Make sure all URL arguments are captured as array. For simple
+# Make sure all URL arguments are captured as array.  For simple
 # search they'll be arrays with only one element at index 0 (this
 # will avoid to deal with scalar in simple search and array in
 # advanced which would greatly complexifies the code).
@@ -166,26 +163,25 @@ foreach ($url_params as $field => $value_id)
     sanitize_field ($field);
     $url_params[$field] = sanitize_value_id ($value_id);
 
-    if (trackers_data_is_date_field ($field))
+    if (!trackers_data_is_date_field ($field))
+      continue;
+    if ($advsrch)
       {
-        if ($advsrch)
-          {
-            $co_field = "{$field}_end";
-            $names =
-              ['preg' => [[$field, $co_field, '/^\d{4}-\d{1,2}-\d{1,2}$/']]];
-          }
-        else
-          {
-            $co_field = "{$field}_op";
-            $names = ['strings' => [[$co_field, trackers_date_op_list ()]]];
-          }
-        $in = sane_import ('request', $names);
-        $url_params[$co_field] = $in[$co_field];
-        if ($advsrch && empty($in[$field]))
-          unset ($url_params[$field]);
-        if (!$advsrch && !$url_params[$co_field])
-          $url_params[$co_field] = ['*'];
+        $co_field = "{$field}_end";
+        $names =
+          ['preg' => [[$field, $co_field, '/^\d{4}-\d{1,2}-\d{1,2}$/']]];
       }
+    else
+      {
+        $co_field = "{$field}_op";
+        $names = ['strings' => [[$co_field, trackers_date_op_list ()]]];
+      }
+    $in = sane_import ('request', $names);
+    $url_params[$co_field] = $in[$co_field];
+    if ($advsrch && empty($in[$field]))
+      unset ($url_params[$field]);
+    if (!$advsrch && !$url_params[$co_field])
+      $url_params[$co_field] = ['*'];
   }
 
 # If history event additional constraint is used, add it.
@@ -225,10 +221,6 @@ if ($order !== null)
       $morder = '';
   }
 
-if ($morder != '' && user_isloggedin ())
-  if ($morder != user_get_preference ($order_pref))
-    user_set_preference ($order_pref, $morder);
-
 # If the report type is not defined then get it from the user preferences.
 # If it is set then update the user preference.  Also initialize the
 # bug report structures.
@@ -255,6 +247,22 @@ if (!trackers_report_init ($report_id))
     trackers_report_init ($report_id);
   }
 
+if ($morder != '' && user_isloggedin ())
+  if ($morder != user_get_preference ($order_pref))
+    user_set_preference ($order_pref, $morder);
+
+$full_field_list = $shown_fields = [];
+while ($field = trackers_list_all_fields ('cmp_place_result'))
+  {
+    $full_field_list[] = $field;
+    if (!trackers_data_is_used ($field))
+      continue;
+    if (!trackers_data_is_showed_on_result ($field))
+      continue;
+    $shown_fields[] = $field;
+  }
+$morder = trackers_filter_morder ($morder, $shown_fields);
+
 function parse_history_field ($val)
 {
   global $history_search, $history_field, $history_event, $history_date;
@@ -323,7 +331,7 @@ function compile_pref_string ($url_params)
         $arr_val = [$arr_val];
       foreach ($arr_val as $value_id)
         if (is_scalar ($value_id))
-          $ret .= "&amp;{$field}[]=$value_id";
+          $ret .= "&amp;{$field}[]=" . urlencode ($value_id);
     }
   $vars =
     ['advsrch', 'msort', 'max_rows', 'spamscore', 'report_id', 'sumORdet'];
@@ -369,11 +377,10 @@ if ($msort === null)
 # system group (meaningful on the cookbook).
 $not_group_specific = 1;
 
-while ($field = trackers_list_all_fields ())
+foreach ($shown_fields as $field)
   {
     # The select boxes for the bug DB search first.
-    if (!(trackers_data_is_showed_on_query ($field)
-          && trackers_data_is_select_box ($field)))
+    if (!trackers_data_is_select_box ($field))
       continue;
     if (!isset ($url_params[$field]))
       $url_params[$field][] = 0;
@@ -384,7 +391,7 @@ while ($field = trackers_list_all_fields ())
 # Force the selection of priority because it is always shown as color code.
 # Force the selection of privacy, we always want to be sure that no private
 # item title is provided to everybody.
-$full_field_list = $col_list = $width_list = $lbl_list = [];
+$col_list = $width_list = $lbl_list = [];
 $select = ['a.group_id', 'a.priority', 'a.privacy', 'a.status_id'];
 
 $where = "WHERE\n    a.group_id = ?";
@@ -450,10 +457,9 @@ foreach ($url_params as $field => $value_id)
       {
         # Transform a date field into a unix time and use <, > or =.
         $param = $url_params[$field][0];
-        list ($time, $ok) = utils_date_to_unixtime ($param);
-        $ok = $ok && preg_match ("/\s*(\d+)-(\d+)-(\d+)/", $param, $match_arr);
+        list ($time, $ok, $date_arr) = utils_date_to_unixtime ($param);
         if ($ok)
-          list (, $year, $month, $day) = $match_arr;
+          list ($year, $month, $day) = $date_arr;
         $field_defined = "\n    AND a.$field <> 0 ";
 
         if ($advsrch)
@@ -486,7 +492,7 @@ foreach ($url_params as $field => $value_id)
                 $where_params[] = $time;
                 $where_params[] = $time_end;
               }
-            else
+            elseif ($operator === '<' || $operator === '>')
               {
                 $time = mktime (0, 0, 0, $month, $day + 1, $year);
                 $where .= "\n    AND a.$field $operator= ?";
@@ -495,10 +501,11 @@ foreach ($url_params as $field => $value_id)
           }
         $where .= $field_defined;
       }
-    elseif ((trackers_data_is_text_field ($field)
-             || trackers_data_is_text_area ($field))
-            && $url_params[$field][0])
+    elseif (trackers_data_is_text ($field))
       {
+        if (!isset ($url_params[$field]))
+          $url_params[$field] = [''];
+
         # Buffer summary and original submission (details) to handle them later
         # in case we have an OR to do between the two, instead of the usual
         # AND.
@@ -639,8 +646,7 @@ while ($field = trackers_list_all_fields ('cmp_place_query'))
           $boxes .= trackers_field_date_operator ($field, $op_value)
             . trackers_field_date ($field, $value);
       }
-    elseif (trackers_data_is_text_field ($field)
-            || trackers_data_is_text_area ($field))
+    elseif (trackers_data_is_text ($field))
       {
         if ($field == 'summary')
           $summary_search = 1;
@@ -657,13 +663,11 @@ while ($field = trackers_list_all_fields ('cmp_place_query'))
       }
     $boxes .= "</span></td>\n";
     $ib++;
-
-  # End of this row.
-  if ($ib % $fields_per_line == 0)
-    {
-      $html_select .= "$labels</tr>\n$boxes</tr>\n";
-      $labels = $boxes = '';
-    }
+    if ($ib % $fields_per_line == 0)
+      {
+        $html_select .= "$labels</tr>\n$boxes</tr>\n";
+        $labels = $boxes = '';
+      }
   } # while ($field = trackers_list_all_fields ('cmp_place_query'))
 
 # Make sure the last few cells are in the table.
@@ -686,18 +690,9 @@ if ($morder != '')
     if (!$msort)
       {
         $matching_morder = preg_replace ('/[<>]$/', '', $morder);
-        while ($field = trackers_list_all_fields ())
-          {
-            if (strcmp ($field, $matching_morder))
-              continue;
-            if (!trackers_data_is_used ($field))
-              continue;
-            if (!trackers_data_is_showed_on_result ($field))
-              continue;
+        foreach ($shown_fields as $field)
+          if (!strcmp ($field, $matching_morder))
             $matching_morder = '';
-            # Can't break here because tracker_list_all_fields ()
-            # only supports full iteration.
-          }
       }
     if (in_array ($matching_morder, ['', 'priority']))
       {
@@ -727,15 +722,8 @@ function lbl_item ($field, $crit)
 $morder_icon_is_set = '';
 $have_last_updated = false;
 $froms = [];
-while ($field = trackers_list_all_fields ('cmp_place_result'))
+foreach ($shown_fields as $field)
   {
-    # Need the full list of used fields
-    $full_field_list[] = $field;
-
-    if (!trackers_data_is_used ($field)
-        || !trackers_data_is_showed_on_result ($field))
-      continue;
-
     if ($field == 'updated')
       $have_last_updated = true;
     $col_list[] = $field;
@@ -795,7 +783,7 @@ while ($field = trackers_list_all_fields ('cmp_place_result'))
     $select[] = "user_$field.user_name AS $field";
     $froms[] = "user user_$field";
     $where .= "\n    AND user_$field.user_id = a.$field";
-  } # while ($field = trackers_list_all_fields ('cmp_place_result'))
+  } # foreach ($shown_fields as $field)
 
 # The submitted_by column may be compared against the user id
 # for access to private items, so make sure it's present.
@@ -1073,9 +1061,7 @@ $form_fieldname = html_build_select_box_from_arrays (
 $form_modified = html_build_select_box_from_arrays (
   $hist_ev_value, $hist_ev_text, 'history_event', $history_event,
   false, '', false, '', false, _("modified or not"));
-$form_since = trackers_field_date (
-  'history_date', $history_date, 0, 0, false
-);
+$form_since = trackers_field_date ('history_date', $history_date);
 if ($form_separator != '')
   $form .= '<p class="smaller"><span class="preinput">'
     # TRANSLATORS: the argument is 'activated' or 'deactivated'.
