@@ -5,7 +5,7 @@
 # Copyright (C) 2000-2006 Mathieu Roy <yeupou--gnu.org>
 # Copyright (C) 2014, 2016, 2017 Assaf Gordon
 # Copyright (C) 2001-2011, 2013, 2017 Sylvain Beucler
-# Copyright (C) 2013, 2014, 2017-2024 Ineiev
+# Copyright (C) 2013, 2014, 2017-2025 Ineiev
 #
 # This file is part of Savane.
 #
@@ -42,11 +42,16 @@
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 require_once (dirname (__FILE__) . '/member.php');
+require_once (dirname (__FILE__) . '/trackers/data.php');
 define ('USER_STATUS_ACTIVE', 'A');
 define ('USER_STATUS_PENDING', 'P');
 define ('USER_STATUS_SQUAD', MEMBER_FLAGS_SQUAD);
 define ('USER_STATUS_SUSPENDED', 'S');
 define ('USER_STATUS_DELETED', 'D');
+define ('USER_EMAIL_PLACEHOLDER', 'idontexist@savane.test');
+define ('USER_NAME_SELF_REMOVED', '-');
+define ('USER_NAME_REMOVED_BY_ADMIN', '-*-');
+define ('USER_NAME_AUTOREMOVED', '-#-');
 
 # User records in this array can be accessed by user_id as well as by user_name.
 # the user_name keys are in the lower case, user names are case-insensitive.
@@ -57,6 +62,18 @@ function user_status_is_removed ($status)
   return $status == USER_STATUS_SUSPENDED || $status == USER_STATUS_DELETED;
 }
 
+function user_suspended_note ($realname)
+{
+  $msg = [
+    USER_NAME_REMOVED_BY_ADMIN => _('The account was removed by an admin.'),
+    USER_NAME_AUTOREMOVED => _('The account was removed automatically.'),
+    USER_NAME_SELF_REMOVED => _('The account was removed by its user.'),
+  ];
+  if (empty ($msg[$realname]))
+    return $realname;
+  return $msg[$realname];
+}
+
 function user_isloggedin ()
 {
   global $G_USER;
@@ -78,8 +95,8 @@ function user_can_be_super_user ($user_id = 0)
   # Admins of sys_group_id are admins and have superuser privs site-wide.
   $result = db_execute ("
     SELECT * FROM user_group
-    WHERE user_id = ? AND group_id = ? AND admin_flags = 'A'",
-    [$user_id, $GLOBALS['sys_group_id']]
+    WHERE user_id = ? AND group_id = ? AND admin_flags = ?",
+    [$user_id, $GLOBALS['sys_group_id'], MEMBER_FLAGS_ADMIN]
   );
   $USER_IS_SUPER_USER = (db_numrows ($result) >= 1);
   return $USER_IS_SUPER_USER;
@@ -98,29 +115,29 @@ function user_ismember ($group_id, $type = 0)
   return member_check (0, $group_id, $type);
 }
 
-# Check the user role in a project  - deprecated.
-function user_check_ismember ($user_id, $group_id, $type = 0)
+function user_is_group_admin ($group = NULL)
 {
-  return member_check ($user_id, $group_id, $type);
+  global $group_id;
+  if ($group === NULL)
+    $group = $group_id;
+  return user_ismember ($group, MEMBER_FLAGS_ADMIN);
 }
 
-# Get the ids of the groups the user belongs to.
-function user_groups ($uid)
+function user_check_group_admin ()
 {
-  $result = db_execute ("SELECT * FROM user_group WHERE user_id = ", [$uid]);
-  $arr = [];
-  while ($val = db_fetch_array ($result))
-    array_push ($arr, $val['group_id']);
-  return $arr;
+  exit_if_no_group ();
+  if (!user_is_group_admin ())
+    exit_permission_denied ();
 }
 
 # Return arrays of ids and names of the groups the user belogs to.
 function user_group_names ($uid)
 {
   $result = db_execute ("
-    SELECT g.group_id, g.unix_group_name, g.group_name
-    FROM `groups` g JOIN user_group u ON g.group_id = u.group_id
-    WHERE u.user_id = ? AND g.status = 'A' ORDER BY g.unix_group_name", [$uid]
+    SELECT `g`.`group_id`, `g`.`unix_group_name`, `g`.`group_name`
+    FROM `groups` `g` JOIN `user_group` `u` ON `g`.`group_id` = `u`.`group_id`
+    WHERE `u`.`user_id` = ? AND `g`.`status` = ? ORDER BY `g`.`unix_group_name`",
+    [$uid, GROUP_STATUS_ACTIVE]
   );
   $ids = $names = [];
   while ($row = db_fetch_array ($result))
@@ -319,7 +336,9 @@ function user_refetch_data ($users = null)
 {
   global $USER_ARR;
   if ($users === null)
-    $users = [user_getid ()];
+    $users = user_getid ();
+  if (!is_array ($users))
+    $users = [$users];
   foreach ($users as $u)
     if (array_key_exists ($u, $USER_ARR))
       unset ($USER_ARR[$u]);
@@ -368,25 +387,21 @@ function user_set_preference ($preference_name, $value)
   global $user_pref;
   if (!user_isloggedin ())
     return false;
-  $preference_name = user_normalize_pref_name ($preference_name);
-  $res = db_execute ("
-    SELECT NULL FROM user_preferences
-    WHERE user_id = ? AND preference_name = ?",
-    [user_getid (), $preference_name]
-  );
-  if (db_numrows ($res) > 0)
-    $result = db_autoexecute ('user_preferences',
+  $pref_name = user_normalize_pref_name ($preference_name);
+  if (user_get_preference ($pref_name, user_getid ()) !== null)
+    db_autoexecute ('user_preferences',
       ['preference_value' => $value], DB_AUTOQUERY_UPDATE,
-      "user_id = ? AND preference_name = ?", [user_getid (), $preference_name]
+      "`user_id` = ? AND `preference_name` = ?", [user_getid (), $pref_name]
     );
   else
-    $result = db_autoexecute ('user_preferences',
-      [ 'user_id' => user_getid (), 'preference_name' => $preference_name,
-        'preference_value' => $value],
-      DB_AUTOQUERY_INSERT
+    db_autoexecute ('user_preferences',
+      [
+        'user_id' => user_getid (), 'preference_name' => $pref_name,
+        'preference_value' => $value
+      ]
     );
   if (isset ($user_pref))
-    $user_pref[$preference_name] = $value;
+    $user_pref[$pref_name] = $value;
   return true;
 }
 
@@ -571,7 +586,7 @@ function user_delete_query_forms ($user_id, $art)
 {
   $res = db_execute (
     "SELECT report_id FROM {$art}_report WHERE scope = ? AND user_id = ?",
-    ['I', $user_id]
+    [QUERY_SCOPE_PERSONAL, $user_id]
   );
   $report_ids = [];
   while ($row = db_fetch_array ($res))
@@ -622,43 +637,82 @@ function user_purge ($user_id)
 {
   db_execute ("DELETE FROM user where user_id = ?", [$user_id]);
   user_delete_aux_data ($user_id);
+  member_rename_in_files (user_getname ($user_id), '_');
 }
 
-# Rename account, with necessary history adjustments in the database (unless
-# that would raise any concerns).
-function user_rename ($user_id, $new_name)
+function user_rename_in_group_history ($old_name, $new_name)
 {
   global $user_history_field_names;
+  db_execute ("
+    UPDATE group_history SET old_value = ?
+    WHERE old_value = ? AND $user_history_field_names",
+    [$new_name, $old_name]
+  );
+}
+
+function user_rename_in_user_group ($old_name, $new_name)
+{
+  db_execute (
+    "UPDATE `user_group` SET `cache_user_name` = ? WHERE `cache_user_name` = ?",
+    [$new_name, $old_name]
+  );
+}
+
+# If duplicate users with $new_name exist in the database,
+# revert user #$user_id to $old_name and return an error message,
+# else return null.
+function user_check_for_duplicate_names ($user_id, $old_name, $new_name)
+{
+  $res = db_execute (
+    "SELECT user_id FROM user WHERE user_name = ?", [$new_name]
+  );
+  if (db_numrows ($res) < 2)
+    return null;
+  db_execute (
+    "UPDATE user SET user_name = ? WHERE user_id = ?", [$old_name, $user_id]
+  );
+  return sprintf ('User <%s> already exists', $new_name);
+}
+
+# Rename account, with necessary adjustments in the database.
+function user_rename ($user_id, $new_name)
+{
   $old_name = user_fetch_name ($user_id);
   if ($old_name == '')
     return sprintf ('No user #%i in the database', $user_id);
   db_execute (
     "UPDATE user SET user_name = ? WHERE user_id = ?", [$new_name, $user_id]
   );
-  $res = db_execute (
-    "SELECT user_id FROM user WHERE user_name = ?", [$new_name]
-  );
-  if (db_numrows ($res) > 1)
-    {
-      db_execute (
-        "UPDATE user SET user_name = ? WHERE user_id = ?",
-        [$old_name, $user_id]
-      );
-      return sprintf ('User <%s> already exists', $new_name);
-    }
-  db_execute ("
-    UPDATE group_history set old_value = ?
-    WHERE old_value = ? AND $user_history_field_names",
-    [$new_name, $old_name]
-  );
+  $error_msg = user_check_for_duplicate_names ($user_id, $old_name, $new_name);
+  if ($error_msg !== null)
+    return $errormsg;
+  user_rename_in_group_history ($old_name, $new_name);
+  user_rename_in_user_group ($old_name, $new_name);
+  if (user_get_field ($user_id, 'uidNumber'))
+    member_rename_in_files ($old_name, $new_name);
   return '';
 }
 
-# Function that should always be used to remove an user account.
-# This function should always be used in a secure context, when user_id
-# is 100% sure.
-# Best is to not to pass the user_id argument unless necessary.
-function user_delete ($user_id = false)
+function user_clear_account_fields ($user_id)
+{
+  $new_realname = USER_NAME_REMOVED_BY_ADMIN;
+  if ($user_id == user_getid ())
+    $new_realname = USER_NAME_SELF_REMOVED;
+  $arg = [
+    'user_pw' => '!', 'realname' => $new_realname, 'confirm_hash' => null,
+    'status' => USER_STATUS_SUSPENDED, 'email' => USER_EMAIL_PLACEHOLDER,
+    'authorized_keys' => '', 'gpg_key' => '', 'people_view_skills' => '0',
+    'people_resume' => '', 'timezone' => 'GMT', 'theme' => '', 'email_new' => ''
+  ];
+  $fail = !db_autoexecute (
+    'user', $arg, DB_AUTOQUERY_UPDATE, "user_id = ?", [$user_id]
+  );
+  if ($fail)
+    fb (_("Failed to update the database."), 1);
+  return $fail;
+}
+
+function user_check_user_id_for_deletion ($user_id)
 {
   if (!$user_id)
     $user_id = user_getid ();
@@ -667,7 +721,15 @@ function user_delete ($user_id = false)
   # and owner of the account.
   if (!user_is_super_user () && $user_id != user_getid ())
     exit_permission_denied ();
+  return $user_id;
+}
 
+# Remove an account.  This function should only be used when logged-in.
+function user_delete ($user_id = false)
+{
+  $user_id = user_check_user_id_for_deletion ($user_id);
+  if (!$user_id)
+    return false;
   if (!user_has_history ($user_id))
     {
       user_purge ($user_id);
@@ -675,22 +737,8 @@ function user_delete ($user_id = false)
       return true;
     }
 
-  $new_realname = '-*-';
-  if ($user_id == user_getid ())
-    $new_realname = '-';
-  $success = db_autoexecute ('user',
-    [ 'user_pw' => '!', 'realname' => $new_realname, 'status' => 'S',
-      'email' => 'idontexist@savane.test', 'confirm_hash' => null,
-      'authorized_keys' => '', 'people_view_skills' => '0',
-      'people_resume' => '', 'timezone' => 'GMT', 'theme' => '',
-      'gpg_key' => '', 'email_new' => ''],
-    DB_AUTOQUERY_UPDATE, "user_id = ?", [$user_id]
-  );
-  if (!$success)
-    {
-      fb (_("Failed to update the database."), 1);
-      return false;
-    }
+  if (user_clear_account_fields ($user_id))
+    return false;
   user_delete_aux_data ($user_id);
   # Rename user; the name starts with '_' so it can't be registered manually,
   # and it shall be unique because it's derived from $user_id.
@@ -725,11 +773,13 @@ function user_list_groups_with_history ($uid, $skip)
 function user_list_groups_without_history ($uid, $skip)
 {
   return db_execute ("
-    SELECT g.group_name, g.group_id, g.unix_group_name, g.is_public,
-      u.admin_flags, 0 as date
-    FROM `groups` g, user_group u
-    WHERE g.group_id = u.group_id AND u.user_id = ? AND g.status = 'A' $skip
-    ORDER BY g.group_id", [$uid]
+    SELECT `g`.`group_name`, `g`.`group_id`, `g`.`unix_group_name`,
+      `g`.`is_public`, `u`.`admin_flags`, 0 as `date`
+    FROM `groups` `g`, `user_group` `u`
+    WHERE
+      `g`.`group_id` = `u`.`group_id` AND `u`.`user_id` = ?
+      AND `g`.`status` = ? $skip
+    ORDER BY `g`.`group_id`", [$uid, GROUP_STATUS_ACTIVE]
   );
 }
 
