@@ -234,35 +234,27 @@ function utils_cutstring ($string, $length = 35)
   return $string;
 }
 
-function utils_utils_strftime ($timestamp, $format)
+function utils_strftime ($timestamp, $format)
 {
-  $env['LC_ALL'] = setlocale (LC_TIME, 0);
-  $env['TZ'] = getenv ('TZ');
-  $cmd = "date +$format -d @$timestamp";
-  utils_run_proc ($cmd, $output, $error, ['env' => $env]);
-  $output = substr ($output, 0, -1); # Drop trailing "\n".
-  return $output;
-}
-
-if (!empty ($sys_use_strftime) && function_exists ("strftime"))
-  {
-    # Used until strftime is dropped from PHP.
-    function utils_strftime ($timestamp, $format)
+  if (!empty ($GLOBALS['sys_use_strftime']) && function_exists ("strftime"))
     {
+      if ($format === null)
+        return "PHP$timestamp";
       $state = utils_disable_warnings (E_DEPRECATED);
       $ret = strftime ($format, $timestamp);
       utils_restore_warnings ($state);
       return $ret;
     }
-  }
-else # !(!empty ($sys_use_strftime) && function_exists ("strftime"))
-  {
-    # Fallback for the PHP versions (> 8.1) when no usable strftime is provided.
-    function utils_strftime ($timestamp, $format)
-    {
-      return utils_utils_strftime ($timestamp, $format);
-    }
-  } # !(!empty ($sys_use_strftime) && function_exists ("strftime"))
+  if ($format === null)
+    return "date-based$timestamp";
+  $env['LC_ALL'] = setlocale (LC_TIME, 0);
+  $env['TZ'] = getenv ('TZ');
+  $cmd = "date +$format -d @$timestamp";
+  utils_run_proc ($cmd, $output, $error, ['env' => $env]);
+  if ($output !== null)
+    return substr ($output, 0, -1); # Drop trailing "\n".
+  return ''; # Most probably, couldn't run $cmd due to lack of memory.
+}
 
 # Return a formatted date for a unix timestamp.
 #
@@ -312,10 +304,10 @@ function utils_date_to_unixtime ($date)
 {
   $res = preg_match ("/\s*(\d+)-(\d+)-(\d+)/", $date, $match_arr);
   if ($res == 0)
-    return [0, false];
+    return [0, false, null];
   list (, $year, $month, $day) = $match_arr;
   $time = mktime (0, 0, 0, $month, $day, $year);
-  return [$time, true];
+  return [$time, true, [$year, $month, $day]];
 }
 
 function utils_read_file ($filename)
@@ -328,24 +320,40 @@ function utils_read_file ($filename)
   return $val;
 }
 
+function utils_ini_size_bytes ($size)
+{
+  $factors = ['k' => 1024, 'm' => 1024 * 1024, 'g' => 1024 * 1024 * 1024];
+  $suff = strtolower (substr ($size, -1));
+  if (!array_key_exists ($suff, $factors))
+    return $size;
+  return substr ($size, 0, -1) * $factors[$suff];
+}
+
+function utils_human_readable_size ($size)
+{
+  if ($size < 1024)
+    # TRANSLATORS: this expresses file size.
+    return sprintf (_("%sB"), round ($size));
+  $size /= 1024;
+  # TRANSLATORS: this expresses file size.
+  $fmt = _("%sKiB");
+  if ($size >= 1024)
+    {
+      # TRANSLATORS: this expresses file size.
+      $fmt = _("%sMiB");
+      $size /= 1024;
+    }
+  $size = sprintf ('%.' . ($size < 10? 1: 0). 'f', $size);
+  return sprintf ($fmt, $size);
+}
+
 function utils_filesize ($filename, $file_size = 0)
 {
   # If file size is defined, assume that we just want an unit conversion.
-
   # Round results: Savane is not a math software.
   if (!isset ($file_size))
     $file_size = filesize ($filename);
-
-  if ($file_size >= 1048576)
-    # TRANSLATORS: this expresses file size.
-    $file_size = sprintf (_("%sMiB"), round ($file_size / 1048576));
-  elseif ($file_size >= 1024)
-    # TRANSLATORS: this expresses file size.
-    $file_size = sprintf(_("%sKiB"), round ($file_size / 1024));
-  else
-    # TRANSLATORS: this expresses file size.
-    $file_size = sprintf(_("%sB"), round ($file_size));
-  return $file_size;
+  return utils_human_readable_size ($file_size);
 }
 
 # Return human-readable sizes.
@@ -707,16 +715,11 @@ function fb_dbresult ($success)
     fb_dberror ();
 }
 
-function utils_help ($text, $explanation_array)
-{
-  return help ($text, $explanation_array);
-}
-
 # Print help about a word.
 #   $text is the sentence where ballons are
 #   $explanation_array is the table word->explanation, must be in the
 #   array syntax.
-function help ($text, $explanation_array)
+function utils_help ($text, $explanation_array)
 {
   if (!is_array ($explanation_array))
     return "<span class='help' title=\"$explanation_array\">$text</span>";
@@ -727,22 +730,9 @@ function help ($text, $explanation_array)
   return $text;
 }
 
-# Remove old $domain-based cookies, see Savannah sr #111062.
-# After 2025-05-13 all such cookies will expire, and this function
-# may be removed.
-function utils_clear_spurious_cookie ($name, $value, $secure = false)
-{
-  global $sys_home, $sys_default_domain;
-  $domain = preg_replace ('/:\d*$/', '', $sys_default_domain);
-  setcookie (
-    $name, $value, time () - 3600, $sys_home, $domain, $secure, true
-  );
-}
-
 function utils_setcookie ($name, $value, $expire, $secure = false)
 {
   global $sys_home;
-  utils_clear_spurious_cookie ($name, $value, $secure);
   setcookie ($name, $value, $expire, $sys_home, '', $secure, true);
 }
 
@@ -769,7 +759,7 @@ function utils_set_csp_headers ()
 } # namespace {
 
 namespace utils_run_proc_ns {
-function dispatch_error ($cmd, $code, $err, $log_error, $in)
+function dispatch_error ($cmd, $code, $err, $log_error, $in = null)
 {
   if (!$code || !$log_error)
     return;
@@ -807,7 +797,7 @@ function p_open ($cmd, $env, $log_error)
   if ($proc !== false)
     return [null, '', $pipes, $proc];
   $err = "can't run $cmd\n";
-  utils_run_proc_ns\dispatch_error ($cmd, -1, $err, $log_error);
+  dispatch_error ($cmd, -1, $err, $log_error);
   return ['fail', $err, null, null];
 }
 
