@@ -42,6 +42,7 @@
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 require_once (dirname (__FILE__) . '/vcs/git.php');
+require_once (dirname (__FILE__) . '/vcs/cvs.php');
 
 function vcs_sorting_sequence ($vcs, $group_id)
 {
@@ -259,8 +260,7 @@ function vcs_print_links_to_repos ($group, $group_id, $vcs)
 
 function vcs_page ($vcs, $group_id)
 {
-  if (!$group_id)
-    exit_no_group ();
+  exit_if_no_group ();
   $vcs_name = vcs_label ($vcs);
   if ($vcs_name === null)
     exit_error ();
