@@ -5,7 +5,7 @@
 # Copyright (C) 2000-2006 Mathieu Roy <yeupou--gnu.org>
 # Copyright (C) 2014, 2016, 2017 Assaf Gordon
 # Copyright (C) 2001-2011, 2013, 2017 Sylvain Beucler
-# Copyright (C) 2013, 2014, 2017-2024 Ineiev
+# Copyright (C) 2013, 2014, 2017-2025 Ineiev
 #
 # This file is part of Savane.
 #
@@ -67,8 +67,7 @@ extract (sane_import ('post',
 ));
 form_check ($submit_buttons);
 
-if (!$group_id)
-  exit_no_group ();
+exit_if_no_group ();
 
 if (!member_check (0, $group_id))
   exit_permission_denied ();
