@@ -5,7 +5,7 @@
 # Copyright (C) 2000-2006 Mathieu Roy <yeupou--gnu.org>
 # Copyright (C) 2014, 2016, 2017 Assaf Gordon
 # Copyright (C) 2001-2011, 2013, 2017 Sylvain Beucler
-# Copyright (C) 2013, 2014, 2017-2024 Ineiev
+# Copyright (C) 2013, 2014, 2017-2025 Ineiev
 #
 # This file is part of Savane.
 #
@@ -79,8 +79,8 @@ if (!$allow_resume)
     $result = db_execute ("
       SELECT g.group_id
       FROM user_group u JOIN groups g ON g.group_id = u.group_id
-      WHERE status = 'A' AND user_id = ? AND admin_flags != ?
-      LIMIT 1", [user_getid (), MEMBER_FLAGS_PENDING]
+      WHERE status = ? AND user_id = ? AND admin_flags != ?
+      LIMIT 1", [USER_STATUS_ACTIVE, user_getid (), MEMBER_FLAGS_PENDING]
     );
     $allow_resume = db_numrows ($result);
   }
@@ -88,7 +88,7 @@ if (!$allow_resume)
 if ($update_profile)
   {
     $arg_arr = [$people_view_skills, user_getid ()];
-    $sql_str = "people_view_skills=?";
+    $sql_str = "people_view_skills = ?";
     if ($allow_resume)
       {
         if (!$people_resume)
@@ -128,10 +128,10 @@ elseif ($update_skill_inventory)
           [$skill_level_id, $skill_year_id, user_getid (), $skill_inventory_id]
         );
 
-        if (!db_affected_rows ($result))
-          fb (_("User Skill update failed"), 1);
-        else
+        if (db_affected_rows ($result))
           fb (_("User Skills updated successfully"));
+        else
+          fb (_("User Skill update failed"), 1);
       }
   }
 elseif ($delete_from_skill_inventory)
@@ -144,10 +144,10 @@ elseif ($delete_from_skill_inventory)
       WHERE user_id = ? AND skill_inventory_id = ?",
       [user_getid (), $skill_inventory_id]
     );
-    if (!db_affected_rows ($result))
-      fb (_("User Skill Delete failed"), 1);
-    else
+    if (db_affected_rows ($result))
       fb (_("User Skill Deleted successfully"));
+    else
+      fb (_("User Skill Delete failed"), 1);
   }
 
 # Fill in the info to edit the resume.
@@ -161,7 +161,7 @@ print '<p>'
 
 $result = db_execute ("SELECT * FROM user WHERE user_id = ?", [user_getid ()]);
 if (!db_numrows ($result))
-  exit_error (_("No such user"));
+  exit_user_not_found (user_getid ());
 utils_get_content ("people/editresume");
 
 print form_tag () . html_h (2, _("Publicly Viewable"))
