@@ -5,7 +5,7 @@
 # Copyright (C) 2000-2006 Mathieu Roy <yeupou--gnu.org>
 # Copyright (C) 2014, 2016, 2017 Assaf Gordon
 # Copyright (C) 2001-2011, 2013, 2017 Sylvain Beucler
-# Copyright (C) 2013, 2014, 2017-2024 Ineiev
+# Copyright (C) 2013, 2014, 2017-2025 Ineiev
 #
 # This file is part of Savane.
 #
@@ -52,11 +52,7 @@ extract (sane_import ('post',
   ]
 ));
 form_check ('update');
-if (!$group_id)
-  exit_no_group ();
-
-if (!user_ismember ($group_id, 'A'))
-  exit_permission_denied ();
+user_check_group_admin ();
 
 $grp = project_get_object ($group_id);
 
