@@ -5,7 +5,7 @@
 # Copyright (C) 2000-2006 Mathieu Roy
 # Copyright (C) 2014, 2016, 2017 Assaf Gordon
 # Copyright (C) 2001-2011, 2013, 2017 Sylvain Beucler
-# Copyright (C) 2013, 2014, 2017-2024 Ineiev
+# Copyright (C) 2013, 2014, 2017-2025 Ineiev
 #
 # This file is part of Savane.
 #
@@ -44,6 +44,8 @@
 require_once('../../include/init.php');
 require_once('../../include/form.php');
 
+session_require (['group' => SESSION_ADMIN_GROUP]);
+
 extract (sane_import ('request', ['true' => ['people_cat', 'people_skills']]));
 extract (sane_import ('post',
   [
@@ -52,11 +54,6 @@ extract (sane_import ('post',
   ]
 ));
 form_check ('post_changes');
-
-# This page is for site admins only.
-if (!user_ismember (1, 'A'))
-  exit_permission_denied ();
-
 function report_db_result ($result, $success_msg)
 {
   if ($result)
