@@ -6,7 +6,7 @@
 # Copyright (C) 2000-2006 Mathieu Roy <yeupou--gnu.org>
 # Copyright (C) 2014, 2016, 2017 Assaf Gordon
 # Copyright (C) 2001-2011, 2013, 2017 Sylvain Beucler
-# Copyright (C) 2013, 2014, 2017-2024 Ineiev
+# Copyright (C) 2013, 2014, 2017-2025 Ineiev
 #
 # This file is part of Savane.
 #
@@ -52,8 +52,8 @@ if (db_numrows ($res_grp) < 1)
   exit_error (_("Invalid Group"));
 
 # If the group isn't active, require being a member of the super-admin group.
-if (db_result ($res_grp, 0, 'status') != 'A')
-  session_require (['group' => 1]);
+if (db_result ($res_grp, 0, 'status') != GROUP_STATUS_ACTIVE)
+  session_require (['group' => SESSION_ADMIN_GROUP]);
 
 session_require (['group' => $group_id]);
 site_project_header (['group' => $group_id, 'context' => 'ahome']);
