@@ -224,13 +224,13 @@ db_autoexecute ('groups',
   [
     'group_name' => utils_specialchars ($form_full_name),
     'unix_group_name' => strtolower ($form_values['unix_name']),
-    'status' => 'P', 'is_public' => 1, 'register_time' => time (),
+    'status' => GROUP_STATUS_PENDING, 'is_public' => 1,
+    'register_time' => time (), 'type' => $group_type,
     'register_purpose' => utils_specialchars ($form_purpose),
     'required_software' => utils_specialchars ($form_required_sw),
     'other_comments' => utils_specialchars ($form_comments),
     'license' => $form_license,
     'license_other' => utils_specialchars ($form_license_other),
-    'type' => $group_type,
   ], DB_AUTOQUERY_INSERT
 );
 $result = db_execute (
@@ -248,7 +248,7 @@ if (db_affected_rows ($result) < 1)
 $user_id = user_getid ();
 
 # Make the current user an admin.
-$result = member_add ($user_id, $group_id, "A");
+$result = member_add ($user_id, $group_id, MEMBER_FLAGS_ADMIN);
 
 if (!$result)
   exit_error (no_i18n ("Setting you as group admin failed"));
