@@ -5,7 +5,7 @@
 # Copyright (C) 2000-2006 Mathieu Roy <yeupou--gnu.org>
 # Copyright (C) 2014, 2016, 2017 Assaf Gordon
 # Copyright (C) 2001-2011, 2013, 2017 Sylvain Beucler
-# Copyright (C) 2013, 2014, 2017-2024 Ineiev
+# Copyright (C) 2013, 2014, 2017-2025 Ineiev
 #
 # This file is part of Savane.
 #
@@ -43,12 +43,12 @@
 
 require_once ('../include/init.php');
 require_once ('../include/form.php');
-session_require (['group' => '1', 'admin_flags' => 'A']);
+session_require (['group' => SESSION_ADMIN_GROUP]);
 
 # We don't internationalize messages in this file because they are
 # for Savannah admins who use English.
 
-extract (sane_import ('get', ['digits' => ['max_rows', 'offset']]));
+html_nextprev_extract_params (100);
 
 function wash_user ()
 {
@@ -81,8 +81,8 @@ function format_user_data ($entry, $i)
     . utils_link ($url, "{$entry['realname']} &lt;{$entry['user_name']}&gt;")
     . "</td>\n<td class='center'>{$entry['spamscore']}</td>\n"
     . "</td>\n<td class='center'>"
-    . utils_link ("$php_self?wash_user_id={$entry['user_id']}#users_results",
-        html_image ('bool/ok.png', ['alt=' => no_i18n ("Wash score")])
+    . utils_link ("$php_self?wash_user_id={$entry['user_id']}#results",
+        html_image ('bool/ok.png', ['alt' => no_i18n ("Wash score")])
       )
     . "</td>\n";
 }
@@ -144,25 +144,24 @@ $title_arr = [
   no_i18n ("Incriminated posts"), no_i18n ("Flagged by")
 ];
 
-if (empty ($max_rows))
-  $max_rows = 50;
+$sql = "FROM user WHERE status = 'A' AND spamscore > 0";
+$result = db_execute ("SELECT count(DISTINCT(user_id)) AS cnt $sql");
+$cnt = 0;
+if (db_numrows ($result))
+  $cnt = db_fetch_array ($result)['cnt'];
 
-if (empty ($offset))
-  $offset = 0;
-$offset = intval ($offset);
-
-$result = db_execute ("
-  SELECT user_name, realname, user_id, spamscore FROM user
-  WHERE status = 'A' AND spamscore > 0
-  ORDER BY spamscore DESC LIMIT ?, ?", [$offset, $max_rows + 1]
-);
-if (!db_numrows ($result))
+if (!$cnt)
   {
     print '<p>' . no_i18n ("No suspects found.") . "</p>\n";
     $HTML->footer ([]);
     exit (0);
   }
 
+$result = db_execute ("
+  SELECT user_name, realname, user_id, spamscore $sql
+  ORDER BY spamscore DESC LIMIT ?, ?", [$offset, $max_rows + 1]
+);
+
 print html_build_list_table_top ($title_arr);
 
 $i = 0;
@@ -185,6 +184,6 @@ while ($entry = db_fetch_array ($result))
   }
 print "</table>\n";
 
-html_nextprev ("$php_self?", $max_rows, $i, "users");
+html_nextprev ("$php_self?", $offset, $max_rows, $cnt);
 $HTML->footer ([]);
 ?>
