@@ -44,7 +44,7 @@
 $testconfig_php = true;
 require_once ("include/ac_config.php");
 $sys_file_domain = '';
-foreach (['database', 'mailman', 'savane-git'] as $inc)
+foreach (['database', 'mailman', 'savane-git', 'trackers/data'] as $inc)
   require_once ("include/$inc.php");
 
 function return_bytes ($v)
@@ -100,14 +100,14 @@ function read_test_key ($algo)
   $dir = dirname (__FILE__) . "/testing/gpg";
   $keys = [];
   foreach (["$algo-pub.asc", "$algo-priv.asc"] as $p)
-   {
-     $k = utils_read_file ("$dir/$p");
-     $keys[] = $k;
-     if (!empty ($k))
-       continue;
-     print "Can't read $dir/$p";
-     return null;
-   }
+    {
+      $k = utils_read_file ("$dir/$p");
+      $keys[] = $k;
+      if (!empty ($k))
+        continue;
+      print "Can't read $dir/$p";
+      return null;
+    }
   return join ("\n", $keys);
 }
 
@@ -347,6 +347,39 @@ function test_sys_upload_dir ()
   return '<b>unlink failed:</b>' . $e['message'];
 }
 
+function test_strftime_alternatives ($t, &$defs)
+{
+  global $sys_use_strftime;
+  if (empty ($sys_use_strftime))
+    {
+      if (!function_exists ('strftime'))
+        return;
+      $sys_use_strftime = 1;
+    }
+  else
+    unset ($sys_use_strftime);
+  $mark = utils_strftime ('', null);
+  $defs["$mark implementation"] =
+    ['strftime-alt-timestamp', utils_strftime ($t, '%c')];
+}
+
+function test_strftime ()
+{
+  global $sys_use_strftime;
+  print html_h (2, "Implementation of strftime");
+  $saved = $sys_use_strftime;
+  $t = time ();
+  $defs = [];
+  list_sysvar ('use_strftime', $defs);
+  $defs['Check for internal strftime'] =
+    function_exists ('strftime')? 'Exists.': 'Absent.';
+  $defs['Used implementation'] = ['strftime-imp10n', utils_strftime ('', null)];
+  $defs["Timestamp $t"] = ['strftime-timestamp', utils_strftime ($t, '%c')];
+  test_strftime_alternatives ($t, $defs);
+  print html_dl ($defs);
+  $sys_use_strftime = $saved;
+}
+
 function test_captcha ()
 {
   global $sys_captchadir;
@@ -445,6 +478,86 @@ function test_mailman ()
       . "in \$sys_mailman_wrapper.</strong></p>\n";
 }
 
+function test_hash_pfx ()
+{
+  global $sys_pw_prefix;
+  $real_pfx = hash_get_pw_prefix ();
+  if (empty ($sys_pw_prefix))
+    return "<b>unset</b><br />\nThe effective value is <b>$real_pfx</b>.";
+  $pfx = $sys_pw_prefix;
+  if ($pfx != $real_pfx)
+    $pfx .= "<br />\n"
+      . "<i>Overridden; the effective value is</i> <b>$real_pfx</b>.";
+  return $pfx;
+}
+function test_hash_cost ()
+{
+  global $sys_pw_rounds;
+  $real_cost = hash_get_pw_cost ();
+  if (empty ($sys_pw_rounds))
+    return "<b>unset</b><br />\nThe effective value is <b>$real_cost</b>.";
+  $cost = $sys_pw_rounds;
+  if ($real_cost != $cost)
+    $cost .= "<br />\n"
+      . "<i>Overridden; the effective value is</i> <b>$real_cost</b>.";
+  return $cost;
+}
+
+function test_hash_algo ($pfx, $prefix, $rounds)
+{
+  global $sys_pw_prefix, $sys_pw_rounds;
+  $plain = 'foo \xff\xaa\xff \x9f\x98\x85';
+  $sys_pw_prefix = $pfx;
+  if ($pfx === $prefix)
+    $sys_pw_rounds = $rounds;
+  else
+    unset ($sys_pw_rounds);
+  $stored = hash_encryptpw ($plain);
+  $ret = "<b>fail</b>";
+  if ($stored === '*0')
+    $ret .= ': algorithm is unsupported';
+  elseif (account_validpw ($stored, $plain))
+    $ret = 'OK';
+  return $ret;
+}
+
+function test_hash_algos ()
+{
+  global $sys_pw_rounds, $sys_pw_prefix, $hash_silent_crypt;
+  list ($prefix, $rounds, $silent) = [
+    $sys_pw_prefix, $sys_pw_rounds, $hash_silent_crypt
+  ];
+  $hash_silent_crypt = true;
+  $defs = [];
+  foreach (hash_supported_pw_prefices () as $pfx)
+    $defs[$pfx] = test_hash_algo ($pfx, $prefix, $rounds);
+  print html_dl ($defs);
+  list ($sys_pw_prefix, $sys_pw_rounds, $hash_silent_crypt) =
+    [$prefix, $rounds, $silent];
+}
+
+function test_hash ()
+{
+  global $sys_use_php_crypt;
+  print html_h (2, 'Stored passwords');
+  $defs = [
+    'Supported algorithms for new hashes' =>
+      join (', ', hash_supported_pw_prefices ()),
+    'Implementation used' => empty ($sys_use_php_crypt)?
+      'libc via the sv_crypt script': 'PHP crypt() function',
+    'sys_pw_prefix' => test_hash_pfx (), 'sys_pw_rounds' => test_hash_cost ()
+  ];
+  print html_dl ($defs);
+  $saved_use = $sys_use_php_crypt;
+  print html_h (3, 'Testing sv_crypt');
+  $sys_use_php_crypt = false;
+  test_hash_algos ();
+  print html_h (3, 'Testing PHP crypt() function');
+  $sys_use_php_crypt = true;
+  test_hash_algos ();
+  $sys_use_php_crypt = $saved_use;
+}
+
 $page = "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\"\n"
       . "  \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">\n";
 $page .=
@@ -540,7 +653,9 @@ foreach ($phptags as $tag => $good)
   if (compare_ini_vals ($tag, $good, $cmp))
     $have_unset = true;
 # Check against minimum sizes.
-$phptags = ['post_max_size' => '3M', 'upload_max_filesize' => '2M'];
+$phptags = [
+  'memory_limit' => '4M', 'post_max_size' => '3M', 'upload_max_filesize' => '2M'
+];
 $cmp = function ($a, $b) { return return_bytes ($a) >= return_bytes ($b); };
 foreach ($phptags as $tag => $good)
   if (compare_ini_vals ($tag, $good, $cmp))
@@ -645,7 +760,7 @@ function test_language ($lang)
       $res = gettext ($str);
       if (($res == $str  && $lang == 'en') || ($res != $str && $lang != 'en'))
         return "$str => $res";
-     }
+    }
   $ret = "<b>Fail.</b> Check <code>locale -a</code> output,\n"
     . "be sure to install language-pack-* in Trisquel";
   return $ret;
@@ -655,7 +770,7 @@ function test_disabled_languages ($loc_list)
 {
   global $sys_linguas;
   $linguas = $sys_linguas;
-  $ret = "<p>Other defined languages: ";
+  $ret = html_h (3, "Other defined languages") . "<p>";
   if (empty ($loc_list))
     return "{$ret}none.</p>\n";
   $ret .= join (", ", array_keys ($loc_list)) . ".</p>\n";
@@ -676,14 +791,15 @@ function get_locales ()
   if ($res)
     return "<p>Can't get locales defined: </p>\n<pre>$err</pre>\n";
   $out = str_replace ("\n", ', ', substr ($out, 0, -1));
-  return "<p>Locales defined: $out.</p>\n";
+  return html_h (3, "Locales defined") . "<p>$out.</p>\n";
 }
 
 function test_i18n ()
 {
   global $sys_linguas, $languages_available;
+  print html_h (2, "I18n");
   $loc_list = $languages_available;
-  $ret = get_locales () . "<p>sys_linguas: $sys_linguas</p>\n";
+  $ret = get_locales () . html_h (3, "sys_linguas") . "<p>$sys_linguas</p>\n";
   $defs = [];
   foreach (explode (':', $sys_linguas) as $lang)
     {
@@ -693,7 +809,7 @@ function test_i18n ()
     }
   $ret .= html_dl ($defs) . test_disabled_languages ($loc_list);
   i18n_setup ("en_US.UTF-8");
-  return $ret;
+  print $ret;
 }
 
 $page .= html_h (2, "Apache environment variables");
@@ -731,9 +847,21 @@ function test_db_fields ($t, $field_func, &$defs)
       $defs["$t.$f"] = $func ($tstruct, $t, $f);
 }
 
+function check_for_db_test_marker ()
+{
+  $table = 'test_marker';
+  $res = db_execute ("SHOW TABLES LIKE '$table'");
+  if (!db_numrows ($res))
+    return;
+  print "<p><strong>Note:</strong> table '$table' exists. "
+    . "<i>This table should only be present in temporary databases "
+    . "created for testing.</i></p>\n";
+}
+
 function test_db_structure ()
 {
   print html_h (2, 'Database structure');
+  check_for_db_test_marker ();
   $defs = [];
   $table_fields = [
     'user' =>
@@ -745,7 +873,141 @@ function test_db_structure ()
     test_db_fields ($t, $field_func, $defs);
   print html_dl ($defs);
 }
+function query_required_field_usage ()
+{
+  $subqueries = [];
+  foreach (utils_get_tracker_list () as $tracker)
+    $subqueries[] = "
+      (
+        SELECT
+          '$tracker' AS `tracker`, `g`.`group_id`, `unix_group_name`,
+          `f`.`bug_field_id`, `field_name`, `label`
+        FROM
+          `groups` `g` JOIN `{$tracker}_field_usage` `u`
+            ON `g`.`group_id` = `u`.`group_id`
+          JOIN `{$tracker}_field` `f` ON `u`.`bug_field_id` = `f`.`bug_field_id`
+        WHERE `f`.`required` = 1 AND `u`.`use_it` = 0
+      )";
 
+  return db_execute (
+    join (' UNION ', $subqueries)
+    . ' ORDER BY `unix_group_name`, `tracker`, `field_name`, `label`'
+  );
+}
+function print_db_values ($res, $columns)
+{
+  $header_row = "<tr>" . html_join_enclosed ($columns, 'th', '') . "</tr>\n";
+  print "<table border='1'>\n$header_row";
+  while ($row = db_fetch_array ($res))
+    {
+      print "<tr>";
+      foreach ($columns as $col)
+        print "<td>{$row[$col]}</td>";
+      print "</tr>\n";
+    }
+  print "$header_row\n</table>\n";
+}
+function explain_unused_required_tracker_fields ($ids)
+{
+  print "<p>Having required fields that aren't used is counter-intuitive "
+    . "and may result in unexpected behavior when configuring trackers.</p>\n";
+  $q = [];
+  foreach ($ids as $tracker => $vals)
+    $q[] = "
+      UPDATE `{$tracker}_field_usage` SET `use_it` = 1
+      WHERE `bug_field_id` IN (" . join (', ', array_unique ($vals)) . ");
+    ";
+  $query = join ("\n", $q);
+  print "<p>You can fix this issue with a query like <code>$query</code></p>\n";
+}
+function list_unused_required_tracker_fields ()
+{
+  print html_h (3, 'Unused required tracker fields');
+  $res = query_required_field_usage ();
+  if (!db_numrows ($res))
+    {
+      print "<p>No such fields found.</p>\n";
+      return;
+    }
+  $columns = [
+   'group_id', 'unix_group_name', 'tracker',
+   'bug_field_id', 'field_name', 'label'
+  ];
+  $ids = [];
+  while ($row = db_fetch_array ($res))
+    $ids[$row['tracker']][] = $row['bug_field_id'];
+  db_data_seek ($res);
+  explain_unused_required_tracker_fields ($ids);
+  print_db_values ($res, $columns);
+}
+function query_duplicate_usage ()
+{
+  $subqueries = [];
+  foreach (utils_get_tracker_list () as $tracker)
+    $subqueries[] =  "
+      (
+        SELECT
+          '$tracker' AS `tracker`, `unix_group_name`, `u`.`group_id`,
+          `u`.`bug_field_id`, `field_name`, `label`, COUNT(*) AS `count`
+        FROM
+          `{$tracker}_field_usage` `u` JOIN `groups` `g`
+            ON `u`.`group_id` = `g`.`group_id`
+          JOIN `{$tracker}_field` `f` ON `f`.`bug_field_id` = `u`.`bug_field_id`
+        GROUP BY
+          `tracker`, `unix_group_name`, `u`.`group_id`, `u`.`bug_field_id`,
+          `field_name`, `label`
+      )";
+  return db_execute ("
+    SELECT * FROM (" . join (' UNION ', $subqueries) . ") `data`
+    WHERE `count` > 1
+    ORDER BY `unix_group_name`, `tracker`, `bug_field_id`"
+  );
+}
+function explain_duplicate_field_usage ($ids)
+{
+  $q = [];
+  foreach ($ids as $tracker => $vals)
+    foreach ($vals as $group_id => $field_ids)
+      $q[] = "
+        SELECT * FROM `{$tracker}_field_usage`
+        WHERE
+          `bug_field_id` IN (" . join (', ', $field_ids) . ")
+          AND `group_id` = $group_id
+        ORDER BY `bug_field_id`
+      ";
+  $queries = html_join_enclosed ($q, ['li', 'code']);
+  print "<p>Generally, every group should have no more than one usage record "
+    . "for every field.  To look into the issue, you can use queries like,</p>"
+    . "\n<ul>$queries</ul>\n";
+  print "<p>Editing field usage should also fix this issue "
+    . "for that field and group.</p>\n";
+}
+function list_duplicate_field_usage ()
+{
+  print html_h (3, 'Duplicate field usage');
+  $columns = [
+    'group_id', 'unix_group_name', 'tracker', 'bug_field_id',
+    'field_name', 'label', 'count'
+  ];
+  $res = query_duplicate_usage ();
+  if (!db_numrows ($res))
+    {
+      print "<p>No duplicate usage found.</p>\n";
+      return;
+    }
+  $ids = [];
+  while ($row = db_fetch_array ($res))
+    $ids[$row['tracker']][$row['group_id']][] = $row['bug_field_id'];
+  db_data_seek ($res);
+  print_db_values ($res, $columns);
+  explain_duplicate_field_usage ($ids);
+}
+function test_db_values ()
+{
+  print html_h (2, 'Database value consistency');
+  list_unused_required_tracker_fields ();
+  list_duplicate_field_usage ();
+}
 function array_add_suff ($arr, $suff)
 {
   if (!is_array ($suff))
@@ -875,8 +1137,9 @@ function test_mysql ()
   if (try_db_connect ())
     return;
   print html_dl (test_mysql_params ());
-  test_db_structure ();
   test_db_features ();
+  test_db_structure ();
+  test_db_values ();
 }
 
 function list_unset_val ($must_be_unset, $value)
@@ -893,24 +1156,34 @@ function list_unset_val ($must_be_unset, $value)
     . "at production servers.</strong>";
 }
 
+function process_sysvar_flags ($tag, $flags)
+{
+  $ret = [];
+  foreach ($flags as $f => $key)
+    $ret[$key] = strstr ($tag, $f) !== false;
+  $tag = str_replace (array_keys ($flags), '', $tag);
+  return [$tag, $ret];
+}
+
 function list_sysvar ($tag, &$defs)
 {
-  $must_be_unset = substr ($tag, 0, 1) === '!';
-  $tag = preg_replace ('/^!/', '', $tag);
+  list ($tag, $flags) = process_sysvar_flags (
+    $tag, ['!' => 'must_be_unset', '*' => 'must_be_hidden']
+  );
   $var = "sys_$tag";
   $value = '<>';
   if (isset ($GLOBALS[$var]))
     $value = utils_specialchars (print_r ($GLOBALS[$var], true));
-  if ($var == "sys_dbpasswd")
+  if ($flags['must_be_hidden'])
     $value = "**************";
-  $defs[$var] = list_unset_val ($must_be_unset, $value);
+  $defs[$var] = list_unset_val ($flags['must_be_unset'], $value);
 }
 
 function output_sysvars ()
 {
   $variables = [
     'dbcharset',
-    'dbhost', 'dbname', 'dbpasswd', 'dbport', 'dbsocket', 'dbuser',
+    'dbhost', 'dbname', '*dbpasswd', 'dbport', 'dbsocket', 'dbuser',
     'default_domain', 'brother_domain', 'file_domain', 'https_host',
     'gpg_name', 'gpg_home', 'graphviz',
     'etc_dir', 'incdir', 'upload_dir', 'url_topdir',
@@ -918,11 +1191,13 @@ function output_sysvars ()
     'mail_admin', 'mail_domain', 'mail_replyto', 'name',
     'min_gidNumber', 'min_uidNumber', 'group_file', 'passwd_file',
     'passwd_common_gid', 'passwd_home_dir', 'passwd_user_shell',
-    'pw_rounds', 'reply_to',
+    'use_php_crypt', 'pw_prefix', 'pw_rounds', 'reply_to',
     'themedefault', 'unix_group_name', 'upload_max', 'use_strftime',
     'watch_anon_posts', 'new_user_watch_days',
     'mailman_wrapper', 'savane_cgit', 'group_file', 'max_items_per_page',
-    '!debug_footer'
+    'registration_text_spam_test', 'registration_captcha', 'ignore_deployed',
+    'log_exits',
+    'error_cc_limit', 'error_cc_period', '!debug_footer'
   ];
   $defs = [];
   foreach ($variables as $tag)
@@ -992,27 +1267,78 @@ function check_varchar ($table_struct, $table, $field, $n = 153)
   return $type;
 }
 
-function test_sysconfigs ()
+function ini_size_note ($former, $latter)
 {
-  include $GLOBALS['sys_conf_file'];
-  test_sysvars ();
-  print "<p><img src='/file?file_id=test.png' alt='Test image'/></p>\n";
-  test_captcha ();
-  test_mailman ();
-  test_mysql ();
+  return "<br /><strong>Note: $former is greater than $latter. "
+    . 'The effective limit will be the latter.</strong>';
+}
 
-  test_gpg ();
-  print html_h (2, "Other tests");
-  $defs = [];
-  test_repos ($defs);
+function test_upload_ini_vals (&$defs)
+{
+  $mem_limit = utils_ini_size_bytes (ini_get ('memory_limit'));
+  $defs['PHP memory_limit setting'] = utils_human_readable_size ($mem_limit);
+  $post_max = utils_ini_size_bytes (ini_get ('post_max_size'));
+  $defs['PHP post_max_size setting'] = utils_human_readable_size ($post_max);
+  if ($mem_limit < $post_max && $mem_limit >= 0)
+    $defs['PHP memory_limit setting'] .=
+      ini_size_note ('post_max_size', 'memory_limit');
+  $umfs = utils_ini_size_bytes (ini_get ('upload_max_filesize'));
+  $defs['PHP upload_max_filesize setting'] = utils_human_readable_size ($umfs);
+  if ($post_max < $umfs)
+    $defs['PHP upload_max_filesize setting'] .=
+      ini_size_note ('upload_max_filesize', 'post_max_size');
+  return $umfs;
+}
+
+function test_uploads (&$defs)
+{
   $defs['sys_upload_dir writability'] =
     ['sys-upload-dir', test_sys_upload_dir ()];
-  $defs['i18n'] = ['i18n', test_i18n ()];
+  $ini_val = test_upload_ini_vals ($defs);
+  list_sysvar ('upload_max', $arr);
+  $val = $arr['sys_upload_max'];
+  if (strstr ($val, '>') === false)
+    {
+      $arr['sys_upload_max'] *= 1024;
+      $val = utils_human_readable_size ($arr['sys_upload_max']);
+    }
+  $defs['sys_upload_max'] = ['sys-upload-max', $val];
+  if (!array_key_exists ('sys_upload_max', $GLOBALS))
+    return;
+  if ($arr['sys_upload_max'] <= $ini_val)
+    return;
+  $defs['sys_upload_max'][1] .=
+    ini_size_note ('$sys_upload_max', 'upload_max_filesize');
+}
+
+function run_other_tests ()
+{
+  $defs = [];
+  test_repos ($defs);
+  test_uploads ($defs);
   print html_dl ($defs);
   print html_h (3, 'Limiting email error report rate', 'error-cc-limit');
   print error_test_cc_limit ();
   print html_h (3, 'Timestamps', 'error-timestamp');
   print '<p>' . error_test_timestamp () . "</p>\n";
+
+}
+
+function test_sysconfigs ()
+{
+  include $GLOBALS['sys_conf_file'];
+  test_sysvars ();
+  print "<p><img src='/file?file_id=test.png' alt='Test image'/></p>\n";
+  test_strftime ();
+  test_captcha ();
+  test_mailman ();
+  test_hash ();
+  test_mysql ();
+
+  test_gpg ();
+  test_i18n ();
+  print html_h (2, "Other tests");
+  run_other_tests ();
 }
 
 $page .= html_h (2, "Configured settings");
