@@ -5,7 +5,7 @@
 # Copyright (C) 2000-2006 Mathieu Roy
 # Copyright (C) 2014, 2016, 2017 Assaf Gordon
 # Copyright (C) 2001-2011, 2013, 2017 Sylvain Beucler
-# Copyright (C) 2013, 2014, 2017-2024 Ineiev
+# Copyright (C) 2013, 2014, 2017-2025 Ineiev
 #
 # This file is part of Savane.
 #
@@ -255,5 +255,14 @@ $out_html = '<p><br />
 </p>
 ';
 
+run_test ($in, $out, $out_html);
+
+$in = "https://a.test/a?a=&amp;b=c\nhttps://a.test/a?a=&amp;b=";
+$out = "https://a.test/a?a=&b=c\nhttps://a.test/a?a=&b=\n";
+$out_html = '<p><a href="https://a.test/a?a=&amp;b=c">https://a.te ... '
+  . "a?a=&amp;b=c</a><br />\n"
+  . '<a href="https://a.test/a?a=&amp;b=">https://a.te ... '
+  . "/a?a=&amp;b=</a><br />\n</p>\n";
+
 run_test ($in, $out, $out_html);
 ?>
